@@ -1,10 +1,12 @@
 /******************************************************************************
- * $Id: cgiutil.h 6428 2007-07-31 19:58:45Z dmorissette $
+ * $Id: cgiutil.h 7998 2008-10-23 03:35:48Z sdlime $
  *
  * Project:  MapServer
  * Purpose:  cgiRequestObj and CGI parsing utility related declarations.
  * Author:   Steve Lime and the MapServer team.
  *
+ * Notes: Portions derived from NCSA HTTPd Server's example CGI programs (util.c). 
+ *
  ******************************************************************************
  * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
@@ -63,6 +65,8 @@ typedef struct
   char *contenttype;
 
   char *postrequest;
+
+  char *httpcookiedata;
 } cgiRequestObj;
       
 
