@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mapcontext.c 8461 2009-01-28 15:26:35Z tomkralidis $
+ * $Id: mapcontext.c 9544 2009-11-11 20:39:16Z assefa $
  *
  * Project:  MapServer
  * Purpose:  OGC Web Map Context implementation
@@ -28,7 +28,7 @@
 
 #include "mapserver.h"
 
-MS_CVSID("$Id: mapcontext.c 8461 2009-01-28 15:26:35Z tomkralidis $")
+MS_CVSID("$Id: mapcontext.c 9544 2009-11-11 20:39:16Z assefa $")
 
 #if defined(USE_WMS_LYR) && defined(USE_OGR)
 
@@ -549,6 +549,11 @@ int msLoadMapContextLayerStyle(CPLXMLNode *psStyle, layerObj *layer,
   sprintf(pszStyle, "wms_style_%s_sld_body", pszStyleName);
 
   psStyleSLDBody = CPLGetXMLNode(psStyle, "SLD.StyledLayerDescriptor");
+  /*some clients such as OpenLayers add a name space, which I believe is wrong but
+   added this additional test for compatibility #3115*/
+  if (psStyleSLDBody == NULL)
+    psStyleSLDBody = CPLGetXMLNode(psStyle, "SLD.sld:StyledLayerDescriptor");
+
   if(psStyleSLDBody != NULL && &(layer->metadata) != NULL)
   {
       pszValue = CPLSerializeXMLTree(psStyleSLDBody);
@@ -2157,7 +2162,10 @@ int msWriteMapContext(mapObj *map, FILE *stream)
           }
 
           if(bDimensionList)
+          {
               msIO_fprintf( stream, "      </DimensionList>\n");
+              bDimensionList = 0;
+          }
 
           msIO_fprintf(stream, "    </Layer>\n");
       }
