@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapcopy.c 8207 2008-12-08 14:50:17Z tbonfort $
+ * $Id: mapcopy.c 9395 2009-10-07 08:47:37Z tbonfort $
  *
  * Project: MapServer
  * Purpose: Functions to allow copying/cloning of maps
@@ -45,7 +45,7 @@
 #include "mapserver.h"
 #include "mapsymbol.h"
 
-MS_CVSID("$Id: mapcopy.c 8207 2008-12-08 14:50:17Z tbonfort $")
+MS_CVSID("$Id: mapcopy.c 9395 2009-10-07 08:47:37Z tbonfort $")
 
 #include "mapcopy.h"
 
@@ -287,7 +287,12 @@ int msCopyLabel(labelObj *dst, labelObj *src)
     MS_COPYSTELEM(buffer);
     MS_COPYSTELEM(antialias);
     MS_COPYSTELEM(wrap);
+    MS_COPYSTELEM(align);
+    MS_COPYSTELEM(maxlength);
     MS_COPYSTELEM(minfeaturesize);
+    
+    MS_COPYSTELEM(minscaledenom);
+    MS_COPYSTELEM(maxscaledenom);
 
     MS_COPYSTELEM(autominfeaturesize);
 
@@ -371,7 +376,13 @@ int msCopyStyle(styleObj *dst, styleObj *src)
     MS_COPYCOLOR(&(dst->maxcolor), &(src->maxcolor));
 
     MS_COPYSTRING(dst->symbolname, src->symbolname);
-
+    MS_COPYSTELEM(patternlength);
+    for(i=0;i<src->patternlength;i++)
+        dst->pattern[i]=src->pattern[i];
+    MS_COPYSTELEM(gap);
+    MS_COPYSTELEM(linejoin);
+    MS_COPYSTELEM(linejoinmaxsize);
+    MS_COPYSTELEM(linecap);
     MS_COPYSTELEM(symbol);
     MS_COPYSTELEM(size);
     MS_COPYSTELEM(minsize);
@@ -390,7 +401,9 @@ int msCopyStyle(styleObj *dst, styleObj *src)
     MS_COPYSTELEM(_geomtransform);
     MS_COPYSTRING(dst->rangeitem, src->rangeitem);
     MS_COPYSTELEM(rangeitemindex);
-
+    MS_COPYSTELEM(outlinewidth);
+    MS_COPYSTELEM(minscaledenom);
+    MS_COPYSTELEM(maxscaledenom);
     /* TODO: add copy for bindings */
 
     return MS_SUCCESS;
