@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapgdal.c 7418 2008-02-29 00:02:49Z nsavard $
+ * $Id: mapgdal.c 9258 2009-08-17 16:22:53Z warmerdam $
  *
  * Project:  MapServer
  * Purpose:  Implementation of support for output using GDAL.
@@ -31,7 +31,7 @@
 #include "mapserver.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapgdal.c 7418 2008-02-29 00:02:49Z nsavard $")
+MS_CVSID("$Id: mapgdal.c 9258 2009-08-17 16:22:53Z warmerdam $")
 
 #ifdef USE_GDAL
 
@@ -82,10 +82,6 @@ void msGDALCleanup( void )
         GDALDestroyDriverManager();
 #endif
 
-#if GDAL_RELEASE_DATE > 20040101
-        CPLFreeConfig();
-#endif
-
         msReleaseLock( TLOCK_GDAL );
 
         bGDALInitialized = 0;
@@ -401,6 +397,19 @@ int msSaveImageGDAL( mapObj *map, imageObj *image, char *filename )
         }
     }
 
+/* -------------------------------------------------------------------- */
+/*  Try to save resolution in the output file.                          */
+/* -------------------------------------------------------------------- */
+    if( image->resolution > 0 )
+    {
+      char res[30];
+
+      sprintf( res, "%lf", image->resolution );
+      GDALSetMetadataItem( hMemDS, "TIFFTAG_XRESOLUTION", res, NULL );
+      GDALSetMetadataItem( hMemDS, "TIFFTAG_YRESOLUTION", res, NULL );
+      GDALSetMetadataItem( hMemDS, "TIFFTAG_RESOLUTIONUNIT", "2", NULL );
+    }
+    
 /* -------------------------------------------------------------------- */
 /*      Create a disk image in the selected output format from the      */
 /*      memory image.                                                   */
