@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapgeos.c 7418 2008-02-29 00:02:49Z nsavard $
+ * $Id: mapgeos.c 9569 2009-11-27 21:16:01Z dmorissette $
  *
  * Project:  MapServer
  * Purpose:  MapServer-GEOS integration.
@@ -628,7 +628,7 @@ char *msGEOSShapeToWKT(shapeObj *shape)
     return NULL;
     
   /* if we have a geometry, we should update it*/
-  msGEOSFreeGeometry(shape->geometry);
+  msGEOSFreeGeometry(shape);
 
   shape->geometry = (GEOSGeom) msGEOSShape2Geometry(shape);
   g = (GEOSGeom) shape->geometry;
@@ -667,6 +667,50 @@ shapeObj *msGEOSBuffer(shapeObj *shape, double width)
 #endif
 }
 
+shapeObj *msGEOSSimplify(shapeObj *shape, double tolerance)
+{
+#if defined(USE_GEOS) && defined(GEOS_HAS_SIMPLIFY)
+  GEOSGeom g1, g2; 
+
+  if(!shape) 
+    return NULL;
+
+  if(!shape->geometry) /* if no geometry for the shape then build one */
+    shape->geometry = (GEOSGeom) msGEOSShape2Geometry(shape);
+
+  g1 = (GEOSGeom) shape->geometry;
+  if(!g1) return NULL;
+  
+  g2 = GEOSSimplify(g1, tolerance);
+  return msGEOSGeometry2Shape(g2);
+#else
+  msSetError(MS_GEOSERR, "GEOS Simplify support is not available.", "msGEOSTopologyPreservingSimplifier()");
+  return NULL;
+#endif
+}
+
+shapeObj *msGEOSTopologyPreservingSimplify(shapeObj *shape, double tolerance)
+{
+#if defined(USE_GEOS) && defined(GEOS_HAS_SIMPLIFY)
+  GEOSGeom g1, g2; 
+
+  if(!shape) 
+    return NULL;
+
+  if(!shape->geometry) /* if no geometry for the shape then build one */
+    shape->geometry = (GEOSGeom) msGEOSShape2Geometry(shape);
+
+  g1 = (GEOSGeom) shape->geometry;
+  if(!g1) return NULL;
+  
+  g2 = GEOSTopologyPreserveSimplify(g1, tolerance);
+  return msGEOSGeometry2Shape(g2);
+#else
+  msSetError(MS_GEOSERR, "GEOS Simplify support is not available.", "msGEOSTopologyPreservingSimplifier()");
+  return NULL;
+#endif
+}
+
 shapeObj *msGEOSConvexHull(shapeObj *shape)
 {
 #ifdef USE_GEOS
