@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mapgraticule.c 9320 2009-09-19 01:18:37Z assefa $
+ * $Id: mapgraticule.c 10105 2010-04-15 12:17:13Z assefa $
  *
  * Project:  MapServer
  * Purpose:  Graticule Renderer
@@ -30,7 +30,7 @@
 #include <assert.h>
 #include "mapproject.h"
 
-MS_CVSID("$Id: mapgraticule.c 9320 2009-09-19 01:18:37Z assefa $")
+MS_CVSID("$Id: mapgraticule.c 10105 2010-04-15 12:17:13Z assefa $")
 
 /**********************************************************************************************************************
  *
@@ -79,8 +79,11 @@ int msGraticuleLayerOpen(layerObj *layer)
   pInfo->dwhichlatitude = -90.0;
   pInfo->dwhichlongitude = -180.0;
   pInfo->bvertical = 1;
-  
-  if( layer->class[0]->label.size == -1 )
+
+  if( layer->numclasses == 0 )
+      msDebug( "GRID layer has no classes, nothing will be rendered.\n" );
+
+  if( layer->numclasses == 0 || layer->class[0]->label.size == -1 )
     pInfo->blabelaxes = 0;
   else
     pInfo->blabelaxes = 1;
@@ -370,6 +373,7 @@ int msGraticuleLayerNextShape(layerObj *layer, shapeObj *shape)
       }
             
       pInfo->ilabelstate = 0;
+      pInfo->dwhichlongitude      += pInfo->dincrementlongitude;
       break;
 
     default:
@@ -453,6 +457,7 @@ int msGraticuleLayerNextShape(layerObj *layer, shapeObj *shape)
       }
             
       pInfo->ilabelstate    = 0;
+      pInfo->dwhichlatitude += pInfo->dincrementlatitude;
       break;
 
     default:
@@ -464,21 +469,17 @@ int msGraticuleLayerNextShape(layerObj *layer, shapeObj *shape)
   /*
    * Increment and move to next arc
    */
-  pInfo->dwhichlatitude += pInfo->dincrementlatitude;
-
-  if(pInfo->dwhichlatitude > pInfo->dendlatitude) {
-    pInfo->dwhichlatitude   = pInfo->dstartlatitude;
-    pInfo->dwhichlongitude      += pInfo->dincrementlongitude;
-
-    if( pInfo->dwhichlongitude > pInfo->dendlongitude && pInfo->bvertical == 0 )
-      return MS_DONE;
-    else if( pInfo->dwhichlongitude > pInfo->dendlongitude ) {
+ 
+  if( pInfo->bvertical && pInfo->dwhichlongitude > pInfo->dendlongitude )   {
       pInfo->dwhichlatitude   = pInfo->dstartlatitude;
-      pInfo->dwhichlongitude   = pInfo->dstartlongitude;
       pInfo->bvertical   = 0;
-    }
   }
 
+  if (pInfo->dwhichlatitude >  pInfo->dendlatitude)
+    return MS_DONE;
+
+   
+
   return MS_SUCCESS;
 }
 
