@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapimagemap.c 8482 2009-01-29 23:34:29Z tamas $
+ * $Id: mapimagemap.c 9472 2009-10-16 18:17:05Z aboudreault $
  *
  * Project:  MapServer
  * Purpose:  Implements imagemap outputformat support.
@@ -38,7 +38,7 @@
 #include <io.h>
 #endif
 
-MS_CVSID("$Id: mapimagemap.c 8482 2009-01-29 23:34:29Z tamas $")
+MS_CVSID("$Id: mapimagemap.c 9472 2009-10-16 18:17:05Z aboudreault $")
 
 #define MYDEBUG 0
 #define DEBUG_IF if (MYDEBUG > 2) 
@@ -293,7 +293,7 @@ DEBUG_IF printf("addImageCache\n<BR>");
  * a pointer to an imageObj structure.
  */
 imageObj *msImageCreateIM(int width, int height, outputFormatObj *format,
-                          char *imagepath, char *imageurl)
+                          char *imagepath, char *imageurl, double resolution, double defresolution)
 {
     imageObj  *image=NULL;
     if (setvbuf(stdout, NULL, _IONBF , 0)){
@@ -330,6 +330,9 @@ DEBUG_IF printf("msImageCreateIM<BR>\n");
             image->height = height;
             image->imagepath = NULL;
             image->imageurl = NULL;
+            image->resolution = resolution;
+            image->resolutionfactor = resolution/defresolution;
+
 	    if( strcasecmp("ON",msGetOutputFormatOption( format, "DXF", "OFF" )) == 0){
 		    dxf = 1;
 		    im_iprintf(&layerStr, "  2\nLAYER\n 70\n  10\n");
@@ -979,16 +982,16 @@ DEBUG_IF printf("msDrawMarkerSymbolIM\n<BR>");
   bc = style->backgroundcolor.pen;
   fc = style->color.pen;
   oc = style->outlinecolor.pen;
-  ox = style->offsetx; /* TODO: add scaling? */
-  oy = style->offsety;
+  ox = style->offsetx*scalefactor;
+  oy = style->offsety*scalefactor;
   if(style->size == -1) {
       size = msSymbolGetDefaultSize( symbol );
       size = MS_NINT(size*scalefactor);
   }
   else
       size = MS_NINT(style->size*scalefactor);
-  size = MS_MAX(size, style->minsize);
-  size = MS_MIN(size, style->maxsize);
+  size = MS_MAX(size, style->minsize*img->resolutionfactor);
+  size = MS_MIN(size, style->maxsize*img->resolutionfactor);
 
   if(style->symbol > symbolset->numsymbols || style->symbol < 0) return; /* no such symbol, 0 is OK */
 /* if(fc<0 && oc<0) return; // nothing to do */
@@ -1195,8 +1198,8 @@ DEBUG_IF printf("msDrawLineSymbolIM<BR>\n");
   }
   else
       size = MS_NINT(style->size*scalefactor);
-  size = MS_MAX(size, style->minsize);
-  size = MS_MIN(size, style->maxsize);
+  size = MS_MAX(size, style->minsize*img->resolutionfactor);
+  size = MS_MIN(size, style->maxsize*img->resolutionfactor);
 
   if(style->symbol > symbolset->numsymbols || style->symbol < 0) return; /* no such symbol, 0 is OK */
   if (suppressEmpty && p->numvalues==0) return;/* suppress area with empty title */
@@ -1408,8 +1411,8 @@ DEBUG_IF printf("msDrawShadeSymbolIM\n<BR>");
   }
   else
       size = MS_NINT(style->size*scalefactor);
-  size = MS_MAX(size, style->minsize);
-  size = MS_MIN(size, style->maxsize);
+  size = MS_MAX(size, style->minsize*img->resolutionfactor);
+  size = MS_MIN(size, style->maxsize*img->resolutionfactor);
 
 /* DEBUG_IF printf ("a"); */
 /* if(fc==-1 && oc!=-1) { // use msDrawLineSymbolIM() instead (POLYLINE) */
@@ -1829,8 +1832,12 @@ int msDrawLabelCacheIM(imageObj* img, mapObj *map)
   labelObj *labelPtr=NULL;
 
   int marker_width, marker_height;
-  int marker_offset_x, marker_offset_y;
+  int marker_offset_x, marker_offset_y, label_offset_x, label_offset_y;
   rectObj marker_rect;
+  int label_mindistance, label_buffer;
+
+  label_mindistance=-1;
+  label_buffer=0;
 
   DEBUG_IF printf("msDrawLabelCacheIM\n<BR>");
   for(priority=MS_MAX_LABEL_PRIORITY-1; priority>=0; priority--) {
@@ -1853,7 +1860,12 @@ int msDrawLabelCacheIM(imageObj* img, mapObj *map)
     if(msGetLabelSize(img,cachePtr->text, labelPtr, &r, &(map->fontset), layerPtr->scalefactor, MS_TRUE,NULL) == -1)
       return(-1);
 
-    if(labelPtr->autominfeaturesize && ((r.maxx-r.minx) > cachePtr->featuresize))
+    label_offset_x = labelPtr->offsetx*layerPtr->scalefactor;
+    label_offset_y = labelPtr->offsety*layerPtr->scalefactor;
+    label_buffer = MS_NINT(labelPtr->buffer*img->resolutionfactor);
+    label_mindistance = MS_NINT(labelPtr->mindistance*img->resolutionfactor);
+
+    if(labelPtr->autominfeaturesize && (cachePtr->featuresize != -1) && ((r.maxx-r.minx) > cachePtr->featuresize))
       continue; /* label too large relative to the feature */
 
     marker_offset_x = marker_offset_y = 0; /* assume no marker */
@@ -1893,14 +1905,14 @@ int msDrawLabelCacheIM(imageObj* img, mapObj *map)
 	  msFreeShape(cachePtr->poly);
 	  cachePtr->status = MS_TRUE; /* assume label *can* be drawn */
 
-	  p = get_metrics(&(cachePtr->point), position, r, (marker_offset_x + labelPtr->offsetx), (marker_offset_y + labelPtr->offsety), labelPtr->angle, labelPtr->buffer, cachePtr->poly);
+	  p = get_metrics(&(cachePtr->point), position, r, (marker_offset_x + label_offset_x), (marker_offset_y + label_offset_y), labelPtr->angle, label_buffer, cachePtr->poly);
 
 	  if(layerPtr->type == MS_LAYER_ANNOTATION && cachePtr->numstyles > 0)
 	    msRectToPolygon(marker_rect, cachePtr->poly); /* save marker bounding polygon */
 
           /* Compare against rendered labels and markers (sets cachePtr->status) */
           msTestLabelCacheCollisions(&(map->labelcache), labelPtr, 
-                                     -1, -1, labelPtr->buffer, cachePtr, priority, l);
+                                     -1, -1, label_buffer, cachePtr, priority, l, label_mindistance, (r.maxx-r.minx));
 
 	  if(cachePtr->status) /* found a suitable place for this label */
 	    break;
@@ -1913,14 +1925,14 @@ int msDrawLabelCacheIM(imageObj* img, mapObj *map)
 	  msFreeShape(cachePtr->poly);
 	  cachePtr->status = MS_TRUE; /* assume label *can* be drawn */
 
-	  p = get_metrics(&(cachePtr->point), j, r, (marker_offset_x + labelPtr->offsetx), (marker_offset_y + labelPtr->offsety), labelPtr->angle, labelPtr->buffer, cachePtr->poly);
+	  p = get_metrics(&(cachePtr->point), j, r, (marker_offset_x + label_offset_x), (marker_offset_y + label_offset_y), labelPtr->angle, label_buffer, cachePtr->poly);
 
 	  if(layerPtr->type == MS_LAYER_ANNOTATION && cachePtr->numstyles > 0)
 	    msRectToPolygon(marker_rect, cachePtr->poly); /* save marker bounding polygon */
 
           /* Compare against rendered labels and markers (sets cachePtr->status) */
           msTestLabelCacheCollisions(&(map->labelcache), labelPtr, 
-                                     -1, -1, labelPtr->buffer, cachePtr, priority, l);
+                                     -1, -1, label_buffer, cachePtr, priority, l,label_mindistance, (r.maxx-r.minx));
 
 	  if(cachePtr->status) /* found a suitable place for this label */
 	    break;
@@ -1934,9 +1946,9 @@ int msDrawLabelCacheIM(imageObj* img, mapObj *map)
       cachePtr->status = MS_TRUE; /* assume label *can* be drawn */
 
       if(labelPtr->position == MS_CC) /* don't need the marker_offset */
-        p = get_metrics(&(cachePtr->point), labelPtr->position, r, labelPtr->offsetx, labelPtr->offsety, labelPtr->angle, labelPtr->buffer, cachePtr->poly);
+        p = get_metrics(&(cachePtr->point), labelPtr->position, r, label_offset_x, label_offset_y, labelPtr->angle, label_buffer, cachePtr->poly);
       else
-        p = get_metrics(&(cachePtr->point), labelPtr->position, r, (marker_offset_x + labelPtr->offsetx), (marker_offset_y + labelPtr->offsety), labelPtr->angle, labelPtr->buffer, cachePtr->poly);
+        p = get_metrics(&(cachePtr->point), labelPtr->position, r, (marker_offset_x + label_offset_x), (marker_offset_y + label_offset_y), labelPtr->angle, label_buffer, cachePtr->poly);
 
       if(layerPtr->type == MS_LAYER_ANNOTATION && cachePtr->numstyles > 0)
 	msRectToPolygon(marker_rect, cachePtr->poly); /* save marker bounding polygon, part of overlap tests */
@@ -1945,7 +1957,7 @@ int msDrawLabelCacheIM(imageObj* img, mapObj *map)
 
         /* Compare against rendered labels and markers (sets cachePtr->status) */
         msTestLabelCacheCollisions(&(map->labelcache), labelPtr, 
-                                   -1, -1, labelPtr->buffer, cachePtr, priority, l);
+                                   -1, -1, label_buffer, cachePtr, priority, l,label_mindistance, (r.maxx-r.minx));
       }
     } /* end position if-then-else */
 
