@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: maplabel.c 8904 2009-04-14 13:29:10Z dmorissette $
+ * $Id: maplabel.c 9462 2009-10-15 09:20:56Z tbonfort $
  *
  * Project:  MapServer
  * Purpose:  Labeling Implementation.
@@ -38,7 +38,7 @@
 
 #include "mapserver.h"
 
-MS_CVSID("$Id: maplabel.c 8904 2009-04-14 13:29:10Z dmorissette $")
+MS_CVSID("$Id: maplabel.c 9462 2009-10-15 09:20:56Z tbonfort $")
 
 
 /**
@@ -52,7 +52,7 @@ MS_CVSID("$Id: maplabel.c 8904 2009-04-14 13:29:10Z dmorissette $")
  * TODO/FIXME: function will produce erroneous/crashing? results
  * if the wrap character is encoded with multiple bytes
  *
- * see http://mapserver.gis.umn.edu/development/rfc/ms-rfc-40/
+ * see http://mapserver.org/development/rfc/ms-rfc-40.html
  * for a summary of how wrap/maxlength interact on the result
  * of the text transformation
  */
@@ -191,7 +191,7 @@ char *msAlignText(mapObj *map, imageObj *image, labelObj *label, char *text) {
          * as the labelSize computing functions return integer bounding boxes. we assume
          * that the integer rounding for such a number of spaces will be negligeable
          * compared to the actual size of thoses spaces */ 
-        if(msGetLabelSize(image, "                ", label,&label_rect, 
+        if(msGetLabelSize(image, ".              .", label,&label_rect, 
                     &map->fontset, 1.0, MS_FALSE,NULL)==-1) { 
             /*error computing label size, we can't continue*/
 
@@ -332,6 +332,13 @@ int msAddLabel(mapObj *map, int layerindex, int classindex, shapeObj *shape, poi
       return(MS_SUCCESS);
   }
 
+  if(map->scaledenom > 0) {
+    if((label->maxscaledenom != -1) && (map->scaledenom >= label->maxscaledenom))
+        return(MS_SUCCESS);
+    if((label->minscaledenom != -1) && (map->scaledenom < label->minscaledenom))
+        return(MS_SUCCESS);
+  }
+
   /* Validate label priority value and get ref on label cache for it */
   if (label->priority < 1)
     label->priority = 1;
@@ -462,7 +469,8 @@ int msAddLabel(mapObj *map, int layerindex, int classindex, shapeObj *shape, poi
 */
 void msTestLabelCacheCollisions(labelCacheObj *labelcache, labelObj *labelPtr, 
                                 int mapwidth, int mapheight, int buffer,
-                                labelCacheMemberObj *cachePtr, int current_priority, int current_label)
+                                labelCacheMemberObj *cachePtr, int current_priority, 
+                                int current_label, int mindistance, double label_size)
 {
     int i, p;
 
@@ -502,8 +510,9 @@ void msTestLabelCacheCollisions(labelCacheObj *labelcache, labelObj *labelPtr,
 
         for(  ; i < cacheslot->numlabels; i++) { 
             if(cacheslot->labels[i].status == MS_TRUE) { /* compare bounding polygons and check for duplicates */
-
-                if((labelPtr->mindistance != -1) && (cachePtr->classindex == cacheslot->labels[i].classindex) && (strcmp(cachePtr->text,cacheslot->labels[i].text) == 0) && (msDistancePointToPoint(&(cachePtr->point), &(cacheslot->labels[i].point)) <= labelPtr->mindistance)) { /* label is a duplicate */
+                /* We add the label_size to the mindistance value when comparing because we do want the mindistance 
+                   value between the labels and not only from point to point. */
+                if((mindistance != -1) && (cachePtr->classindex == cacheslot->labels[i].classindex) && (strcmp(cachePtr->text,cacheslot->labels[i].text) == 0) && (msDistancePointToPoint(&(cachePtr->point), &(cacheslot->labels[i].point)) <= (mindistance + label_size))) { /* label is a duplicate */
                     cachePtr->status = MS_FALSE;
                     return;
                 }
@@ -657,9 +666,16 @@ int msLoadFontSet(fontSetObj *fontset, mapObj *map)
 
 int msGetTruetypeTextBBox(imageObj *img, char *font, double size, char *string, rectObj *rect, double **advances) {
 #ifdef USE_GD_FT
+	if(img!=NULL && MS_RENDERER_PLUGIN(img->format)) {
+		img->format->vtable->getTruetypeTextBBox(img,font,size,string,rect,advances);
+		//printf("%s: %f %f %f %f\n",string,rect->minx,rect->miny,rect->maxx,rect->maxy);
+		return MS_SUCCESS;
+	} else 
 #ifdef USE_AGG
-    if(img!=NULL && MS_RENDERER_AGG(img->format)) {
-        return msGetTruetypeTextBBoxAGG(img,font,size,string,rect,advances); 
+	if(img!=NULL && MS_RENDERER_AGG(img->format)) {
+        msGetTruetypeTextBBoxAGG(img,font,size,string,rect,advances);
+        //printf("%s: %f %f %f %f\n",string,rect->minx,rect->miny,rect->maxx,rect->maxy);
+        return MS_SUCCESS;
     } else
 #endif
     {
@@ -763,9 +779,15 @@ int msGetLabelSize(imageObj *img, char *string, labelObj *label, rectObj *rect,
     char *font=NULL;
 
     size = label->size*scalefactor;
-    size = MS_MAX(size, label->minsize);
-    size = MS_MIN(size, label->maxsize);
-
+    if (img != NULL) {
+      size = MS_MAX(size, label->minsize*img->resolutionfactor);
+      size = MS_MIN(size, label->maxsize*img->resolutionfactor);
+    }
+    else {
+      size = MS_MAX(size, label->minsize);
+      size = MS_MIN(size, label->maxsize);
+    }
+    scalefactor = size / label->size;
     font = msLookupHashTable(&(fontset->fonts), label->font);
     if(!font) {
       if(label->font) 
@@ -782,15 +804,15 @@ int msGetLabelSize(imageObj *img, char *string, labelObj *label, rectObj *rect,
     if(adjustBaseline) {
       int nNewlines = msCountChars(string,'\n');
       if(!nNewlines) {
-        label->offsety += MS_NINT(((rect->miny + rect->maxy) + size) / 2);
-        label->offsetx += MS_NINT(rect->minx / 2);
+        label->offsety += (MS_NINT(((rect->miny + rect->maxy) + size) / 2))/scalefactor;
+        label->offsetx += (MS_NINT(rect->minx / 2))/scalefactor;
       }
       else {
         rectObj rect2; /*bbox of first line only*/
         char* firstLine = msGetFirstLine(string);
         msGetTruetypeTextBBox(img,font,size,firstLine,&rect2,NULL);
-        label->offsety += MS_NINT(((rect2.miny+rect2.maxy) + size) / 2);
-        label->offsetx += MS_NINT(rect2.minx / 2);
+        label->offsety += (MS_NINT(((rect2.miny+rect2.maxy) + size) / 2))/scalefactor;
+        label->offsetx += (MS_NINT(rect2.minx / 2))/scalefactor;
         free(firstLine);
       }
     }
@@ -1034,7 +1056,7 @@ int msImageTruetypeArrow(symbolSetObj *symbolset, gdImagePtr img, shapeObj *p, s
 #endif
 }
 
-int msImageTruetypePolyline(symbolSetObj *symbolset, gdImagePtr img, shapeObj *p, styleObj *style, double scalefactor)
+int msImageTruetypePolyline(symbolSetObj *symbolset, imageObj *img, shapeObj *p, styleObj *style, double scalefactor)
 {
 #ifdef USE_GD_FT
   int i,j;
@@ -1060,10 +1082,10 @@ int msImageTruetypePolyline(symbolSetObj *symbolset, gdImagePtr img, shapeObj *p
   }
   else
       size = style->size;
-  if(size*scalefactor > style->maxsize) scalefactor = (float)style->maxsize/(float)size;
-  if(size*scalefactor < style->minsize) scalefactor = (float)style->minsize/(float)size;
+  if(size*scalefactor > style->maxsize*img->resolutionfactor) scalefactor = (float)style->maxsize*img->resolutionfactor/(float)size;
+  if(size*scalefactor < style->minsize*img->resolutionfactor) scalefactor = (float)style->minsize*img->resolutionfactor/(float)size;
   gap = MS_ABS(symbol->gap)* (int) scalefactor;
-  label.size = (size * scalefactor);
+  label.size = size ;/* "* scalefactor" removed: this is already scaled in msDrawTextGD()*/
   /* label.minsize = style->minsize; */
   /* label.maxsize = style->maxsize; */
 
