@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: maplegend.c 9432 2009-10-09 13:07:50Z tbonfort $
+ * $Id: maplegend.c 10015 2010-03-25 15:49:05Z assefa $
  *
  * Project:  MapServer
  * Purpose:  Legend generation.
@@ -29,7 +29,7 @@
 
 #include "mapserver.h"
 
-MS_CVSID("$Id: maplegend.c 9432 2009-10-09 13:07:50Z tbonfort $")
+MS_CVSID("$Id: maplegend.c 10015 2010-03-25 15:49:05Z assefa $")
 
 #define PSF .8
 #define VMARGIN 5 /* margin at top and bottom of legend graphic */
@@ -121,13 +121,25 @@ int msDrawLegendIcon(mapObj *map, layerObj *lp, classObj *theclass,
     if(theclass->keyimage != NULL) {
       int symbolNum;
       styleObj imgStyle;
+      symbolObj *symbol=NULL;
       symbolNum = msAddImageSymbol(&(map->symbolset), msBuildPath(szPath, map->mappath, theclass->keyimage));
       if(symbolNum == -1) { 
           msSetError(MS_GDERR, "Failed to open legend key image", "msCreateLegendIcon()");
           return(MS_FAILURE);
       }
+      
+      symbol = map->symbolset.symbol[symbolNum];
+      
       initStyle(&imgStyle);
-      imgStyle.size = width;
+      /*set size so thet symbol will be scaled properly #3296*/
+      if (width/symbol->sizex < height/symbol->sizey)
+        imgStyle.size = symbol->sizey*(width/symbol->sizex);
+      else
+        imgStyle.size = symbol->sizey*(height/symbol->sizey);
+
+      if (imgStyle.size > imgStyle.maxsize)
+        imgStyle.maxsize = imgStyle.size;
+
       imgStyle.symbol = symbolNum;
       msDrawMarkerSymbol(&map->symbolset,image,&marker,&imgStyle,lp->scalefactor);
       /* TO DO: we may want to handle this differently depending on the relative size of the keyimage */
