@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapobject.c 8015 2008-11-11 18:14:14Z pramsey $
+ * $Id: mapobject.c 9323 2009-09-21 06:26:37Z sdlime $
  *
  * Project:  MapServer
  * Purpose:  Functions for operating on a mapObj that don't belong in a
@@ -35,7 +35,7 @@
 #  include "cpl_conv.h"
 #endif
 
-MS_CVSID("$Id: mapobject.c 8015 2008-11-11 18:14:14Z pramsey $")
+MS_CVSID("$Id: mapobject.c 9323 2009-09-21 06:26:37Z sdlime $")
 
 void freeWeb(webObj *web);
 void freeScalebar(scalebarObj *scalebar);
@@ -79,9 +79,9 @@ void msFreeMap(mapObj *map) {
 
   if(!map) return;
 
-  /*printf("msFreeMap(): maybe freeing map at %p count=%d.\n",map, map->refcount);*/
-  if ( MS_REFCNT_DECR_IS_NOT_ZERO(map) ) { return; }
-  if (map->debug >= MS_DEBUGLEVEL_VV)
+  /* printf("msFreeMap(): maybe freeing map at %p count=%d.\n",map, map->refcount); */
+  if(MS_REFCNT_DECR_IS_NOT_ZERO(map)) { return; }
+  if(map->debug >= MS_DEBUGLEVEL_VV)
      msDebug("msFreeMap(): freeing map at %p.\n",map);
   
   msCloseConnections(map);
@@ -95,17 +95,7 @@ void msFreeMap(mapObj *map) {
 
   msFreeLabelCache(&(map->labelcache));
 
-  if( map->outputformat && --map->outputformat->refcount < 1 )
-      msFreeOutputFormat( map->outputformat );
-
-  for(i=0; i < map->numoutputformats; i++ ) {
-      if( --map->outputformatlist[i]->refcount < 1 )
-      msFreeOutputFormat( map->outputformatlist[i] );
-  }
-  if( map->outputformatlist != NULL )
-      msFree( map->outputformatlist );
-
-  msFree( map->imagetype );
+  msFree(map->imagetype);
 
   msFreeFontSet(&(map->fontset));
 
@@ -119,21 +109,32 @@ void msFreeMap(mapObj *map) {
   freeLegend(&(map->legend));  
 
   for(i=0; i<map->maxlayers; i++) {
-    if (GET_LAYER(map, i)!=NULL) {
-    	GET_LAYER(map, i)->map=NULL;
-    	if ( freeLayer((GET_LAYER(map, i))) == MS_SUCCESS) {
-	    	free(GET_LAYER(map, i));
-	}
+    if(GET_LAYER(map, i) != NULL) {
+      GET_LAYER(map, i)->map = NULL;
+      if(freeLayer((GET_LAYER(map, i))) == MS_SUCCESS)
+        free(GET_LAYER(map, i));
     }
   }
   msFree(map->layers);
 
-  if (map->layerorder)
-      free(map->layerorder);
+  if(map->layerorder)
+    free(map->layerorder);
 
   msFree(map->templatepattern);
   msFree(map->datapattern);
   msFreeHashItems(&(map->configoptions));
+  if(map->outputformat && map->outputformat->refcount > 0 && --map->outputformat->refcount < 1)
+    msFreeOutputFormat(map->outputformat);
+
+  for(i=0; i<map->numoutputformats; i++ ) {
+    if(map->outputformatlist[i]->refcount > 0 && --map->outputformatlist[i]->refcount < 1)
+      msFreeOutputFormat(map->outputformatlist[i]);
+  }
+  if(map->outputformatlist != NULL)
+    msFree(map->outputformatlist);
+
+  msFreeQuery(&(map->query));
+
   msFree(map);
 }
 
