@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mappostgis.c 9312 2009-09-15 22:34:46Z pramsey $
+ * $Id: mappostgis.c 9967 2010-03-22 13:18:57Z dmorissette $
  *
  * Project:  MapServer
  * Purpose:  PostGIS CONNECTIONTYPE support.
@@ -68,7 +68,7 @@
 
 #ifdef USE_POSTGIS
 
-MS_CVSID("$Id: mappostgis.c 9312 2009-09-15 22:34:46Z pramsey $")
+MS_CVSID("$Id: mappostgis.c 9967 2010-03-22 13:18:57Z dmorissette $")
 
 /*
 ** msPostGISCloseConnection()
@@ -394,6 +394,62 @@ static void find_bounds(shapeObj *shape)
 /* TODO Review and clean above! */
 
 
+static int msPostGISRetrievePgVersion(PGconn *pgconn) { 
+#ifndef POSTGIS_HAS_SERVER_VERSION 
+    int pgVersion = 0; 
+    char *strVersion = NULL; 
+    char *strParts[3] = { NULL, NULL, NULL }; 
+    int i = 0, j = 0, len = 0; 
+    int factor = 10000; 
+ 
+    if (pgconn == NULL) { 
+        msSetError(MS_QUERYERR, "Layer does not have a postgis connection.", "msPostGISRetrievePgVersion()"); 
+        return(MS_FAILURE); 
+    } 
+
+    if (! PQparameterStatus(pgconn, "server_version") )
+        return(MS_FAILURE); 
+ 
+    strVersion = strdup(PQparameterStatus(pgconn, "server_version")); 
+    if( ! strVersion ) 
+        return MS_FAILURE; 
+    strParts[j] = strVersion; 
+    j++; 
+    len = strlen(strVersion); 
+    for( i = 0; i < len; i++ ) { 
+        if( strVersion[i] == '.' ) { 
+            strVersion[i] = '\0'; 
+ 
+            if( j < 3 ) { 
+                strParts[j] = strVersion + i + 1; 
+                j++; 
+            } 
+            else { 
+                free(strVersion); 
+                msSetError(MS_QUERYERR, "Too many parts in version string.", "msPostGISRetrievePgVersion()"); 
+                return MS_FAILURE; 
+            } 
+        } 
+    } 
+     
+    for( j = 0; j < 3 && strParts[j]; j++ ) { 
+        if( atoi(strParts[j]) ) { 
+            pgVersion += factor * atoi(strParts[j]); 
+        } 
+        else { 
+            free(strVersion); 
+            msSetError(MS_QUERYERR, "Unable to parse version string.", "msPostGISRetrievePgVersion()"); 
+            return MS_FAILURE; 
+        } 
+        factor = factor / 100; 
+    } 
+    free(strVersion); 
+    return pgVersion; 
+#else 
+    return PQserverVersion(pgconn); 
+#endif 
+} 
+
 /*
 ** msPostGISRetrievePK()
 **
@@ -439,7 +495,7 @@ int msPostGISRetrievePK(layerObj *layer) {
         msSetError(MS_QUERYERR, "Layer does not have a postgis connection.", "msPostGISRetrievePK()");
         return(MS_FAILURE);
     }
-    pgVersion = PQserverVersion(layerinfo->pgconn);
+    pgVersion = msPostGISRetrievePgVersion(layerinfo->pgconn);
 
     if (pgVersion < 70000) {
         if (layer->debug) {
@@ -856,6 +912,7 @@ int msPostGISBase64Decode(unsigned char *dest, const char *src, int srclen) {
 char *msPostGISBuildSQLBox(layerObj *layer, rectObj *rect, char *strSRID) {
 
     char *strBox = NULL;
+    size_t sz;
 
     if (layer->debug) {
         msDebug("msPostGISBuildSQLBox called.\n");
@@ -864,24 +921,35 @@ char *msPostGISBuildSQLBox(layerObj *layer, rectObj *rect, char *strSRID) {
     if ( strSRID ) {
         static char *strBoxTemplate = "GeomFromText('POLYGON((%.15g %.15g,%.15g %.15g,%.15g %.15g,%.15g %.15g,%.15g %.15g))',%s)";
         /* 10 doubles + 1 integer + template characters */
-        strBox = (char*)malloc(10 * 15 + strlen(strSRID) + strlen(strBoxTemplate));
-        sprintf(strBox, strBoxTemplate,
+        sz = 10 * 22 + strlen(strSRID) + strlen(strBoxTemplate);
+        strBox = (char*)malloc(sz+1); /* add space for terminating NULL */
+        if ( sz <= snprintf(strBox, sz, strBoxTemplate,
                 rect->minx, rect->miny,
                 rect->minx, rect->maxy,
                 rect->maxx, rect->maxy,
                 rect->maxx, rect->miny,
                 rect->minx, rect->miny,
-                strSRID);
+                strSRID) )
+	{
+        	msSetError(MS_MISCERR,"Bounding box digits truncated.","msPostGISBuildSQLBox");
+        	return 0;
+	}
     } else {
         static char *strBoxTemplate = "GeomFromText('POLYGON((%.15g %.15g,%.15g %.15g,%.15g %.15g,%.15g %.15g,%.15g %.15g))')";
         /* 10 doubles + template characters */
-        strBox = (char*)malloc(10 * 15 + strlen(strBoxTemplate));
-        sprintf(strBox, strBoxTemplate,
+        sz = 10 * 22 + strlen(strBoxTemplate);
+        strBox = (char*)malloc(sz+1); /* add space for terminating NULL */
+        if ( sz <= snprintf(strBox, sz, strBoxTemplate,
                 rect->minx, rect->miny,
                 rect->minx, rect->maxy,
                 rect->maxx, rect->maxy,
                 rect->maxx, rect->miny,
-                rect->minx, rect->miny);
+                rect->minx, rect->miny) )
+	{
+        	msSetError(MS_MISCERR,"Bounding box digits truncated.","msPostGISBuildSQLBox");
+        	return 0;
+	}
+	
     }
 
     return strBox;
@@ -1295,7 +1363,7 @@ char *msPostGISBuildSQL(layerObj *layer, rectObj *rect, long *uid) {
 
 }
 
-int msPostGISReadShape(layerObj *layer, shapeObj *shape, int random) {
+int msPostGISReadShape(layerObj *layer, shapeObj *shape) {
 
     char *wkbstr = NULL;
     unsigned char *wkb = NULL;
@@ -1393,26 +1461,21 @@ int msPostGISReadShape(layerObj *layer, shapeObj *shape, int random) {
             }
         }
         
-        if( random ) {
-            /* t is the geometry, t+1 is the uid */
-            tmp = PQgetvalue(layerinfo->pgresult, layerinfo->rownum, t + 1);
-            if( tmp ) {
-                uid = strtol( tmp, NULL, 10 );
-            }
-            else {
-                uid = 0;
-            }
-            if( layer->debug > 4 ) {
-                msDebug("msPostGISReadShape: Setting shape->index = %d\n", uid);
-            }
-            shape->index = uid;
-        } else {
-            if( layer->debug > 4 ) {
-                msDebug("msPostGISReadShape: Setting shape->index = %d\n", layerinfo->rownum);
-            }
-            shape->index = layerinfo->rownum;
+        /* t is the geometry, t+1 is the uid */
+        tmp = PQgetvalue(layerinfo->pgresult, layerinfo->rownum, t + 1);
+        if( tmp ) {
+            uid = strtol( tmp, NULL, 10 );
         }
-
+        else {
+            uid = 0;
+        }
+        if( layer->debug > 4 ) {
+            msDebug("msPostGISReadShape: Setting shape->index = %d\n", uid);
+            msDebug("msPostGISReadShape: Setting shape->tileindex = %d\n", layerinfo->rownum);
+        }
+        shape->index = uid;
+        shape->tileindex = layerinfo->rownum;
+        
         if( layer->debug > 2 ) {
             msDebug("msPostGISReadShape: [index] %d\n",  shape->index);
         }
@@ -1777,7 +1840,7 @@ int msPostGISLayerNextShape(layerObj *layer, shapeObj *shape) {
         if (layerinfo->rownum < PQntuples(layerinfo->pgresult)) {
             int rv;
             /* Retrieve this shape, cursor access mode. */
-            rv = msPostGISReadShape(layer, shape, 0);
+            rv = msPostGISReadShape(layer, shape);
             if( shape->type != MS_SHAPE_NULL ) {
                 (layerinfo->rownum)++; /* move to next shape */
                 return MS_SUCCESS;
@@ -1819,7 +1882,12 @@ int msPostGISLayerResultsGetShape(layerObj *layer, shapeObj *shape, int tile, lo
     assert(layer->layerinfo != NULL);
 
     if (layer->debug) {
-        msDebug("msPostGISLayerGetShape called for record = %i\n", record);
+        msDebug("msPostGISLayerResultsGetShape called for record = %i\n", record);
+    }
+
+    if( tile < 0 ) {
+        msDebug("msPostGISLayerResultsGetShape called for record = %i\n", record);
+        return msPostGISLayerResultsGetShape(layer, shape, tile, record);
     }
 
     layerinfo = (msPostGISLayerInfo*) layer->layerinfo;
@@ -1844,21 +1912,21 @@ int msPostGISLayerResultsGetShape(layerObj *layer, shapeObj *shape, int tile, lo
     }
 
     /* Check the validity of the requested record number. */
-    if( record >= PQntuples(pgresult) ) {
-        msDebug("msPostGISLayerResultsGetShape got record request (%d) but only has %d tuples.\n", record, PQntuples(pgresult));
+    if( tile >= PQntuples(pgresult) ) {
+        msDebug("msPostGISLayerResultsGetShape got request for (%d) but only has %d tuples.\n", tile, PQntuples(pgresult));
         msSetError( MS_MISCERR,
                     "Got request larger than result set.",
                     "msPostGISLayerResultsGetShape()");
         return MS_FAILURE;
     }
 
-    layerinfo->rownum = record; /* Only return one result. */
+    layerinfo->rownum = tile; /* Only return one result. */
 
     /* We don't know the shape type until we read the geometry. */
     shape->type = MS_SHAPE_NULL;
 
 	/* Return the shape, cursor access mode. */
-    result = msPostGISReadShape(layer, shape, 0);
+    result = msPostGISReadShape(layer, shape);
 
     return (shape->type == MS_SHAPE_NULL) ? MS_FAILURE : MS_SUCCESS;
 #else
@@ -1945,7 +2013,7 @@ int msPostGISLayerGetShape(layerObj *layer, shapeObj *shape, int tile, long reco
 
     if (num_tuples > 0) {
         /* Get shape in random access mode. */
-        result = msPostGISReadShape(layer, shape, 1);
+        result = msPostGISReadShape(layer, shape);
     }
 
     return (shape->type == MS_SHAPE_NULL) ? MS_FAILURE : ( (num_tuples > 0) ? MS_SUCCESS : MS_DONE );
