@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mappostgresql.c 9049 2009-05-28 17:58:22Z pramsey $
+ * $Id: mappostgresql.c 9266 2009-08-25 03:13:40Z sdlime $
  *
  * Project:  MapServer
  * Purpose:  Postgres CONNECTIONTYPE support.
@@ -28,7 +28,7 @@
  ****************************************************************************/
 
 
-/* $Id: mappostgresql.c 9049 2009-05-28 17:58:22Z pramsey $ */
+/* $Id: mappostgresql.c 9266 2009-08-25 03:13:40Z sdlime $ */
 #include <assert.h>
 #include "mapserver.h"
 #include "maptime.h"
@@ -50,7 +50,7 @@
 #include <string.h>
 #include <ctype.h> /* tolower() */
 
-MS_CVSID("$Id: mappostgresql.c 9049 2009-05-28 17:58:22Z pramsey $")
+MS_CVSID("$Id: mappostgresql.c 9266 2009-08-25 03:13:40Z sdlime $")
 
 
 typedef struct {
@@ -400,9 +400,7 @@ int msPOSTGRESQLJoinClose(joinObj *join) {
       joininfo->conn = NULL;
     }
 
-    if(joininfo->to_column) {
-      free(joininfo->to_column);
-    }
+    /* removed free(joininfo->to_column), see bug #2936 */
 
     if(joininfo->from_value) {
       free(joininfo->from_value);
