@@ -1,4 +1,5 @@
 /******************************************************************************
+ * $Id: mapproject.c 8022 2008-11-19 01:27:37Z warmerdam $
  *
  * Project:  MapServer
  * Purpose:  projectionObj / PROJ.4 interface.
@@ -28,9 +29,10 @@
 
 #include "mapserver.h"
 #include "mapproject.h"
+#include "mapthread.h"
 #include <assert.h>
 
-MS_CVSID("$Id: mapproject.c 6428 2007-07-31 19:58:45Z dmorissette $")
+MS_CVSID("$Id: mapproject.c 8022 2008-11-19 01:27:37Z warmerdam $")
 
 #ifdef USE_PROJ
 static int msTestNeedWrap( pointObj pt1, pointObj pt2, pointObj pt2_geo,
@@ -86,8 +88,10 @@ int msProjectPoint(projectionObj *in, projectionObj *out, pointObj *point)
           point->y *= DEG_TO_RAD;
       }
 
+      msAcquireLock( TLOCK_PROJ );
       error = pj_transform( in->proj, out->proj, 1, 0, 
                             &(point->x), &(point->y), &z );
+      msReleaseLock( TLOCK_PROJ );
 
       if( error || point->x == HUGE_VAL || point->y == HUGE_VAL )
           return MS_FAILURE;
@@ -873,6 +877,8 @@ void msSetPROJ_LIB( const char *proj_lib )
 #ifdef USE_PROJ
     static int finder_installed = 0;
 
+    msAcquireLock( TLOCK_PROJ );
+
     if( finder_installed == 0 && proj_lib != NULL)
     {
         finder_installed = 1;
@@ -895,6 +901,8 @@ void msSetPROJ_LIB( const char *proj_lib )
 
     if( proj_lib != NULL )
         ms_proj_lib = strdup( proj_lib );
+    
+    msReleaseLock( TLOCK_PROJ );
 #endif
 }
 
@@ -950,3 +958,64 @@ char *msGetProjectionString(projectionObj *proj)
     return pszProjString;
 }
 
+/************************************************************************/
+/*                       msAxisNormalizePoints()                        */
+/*                                                                      */
+/*      Convert the passed points to "easting, northing" axis           */
+/*      orientation if they are not already.                            */
+/************************************************************************/
+
+void msAxisNormalizePoints( projectionObj *proj, int count, 
+                            double *x, double *y )
+
+{
+    int i;
+    const char *axis = NULL;
+
+    for( i = 0; i < proj->numargs; i++ )
+    {
+        if( strstr(proj->args[i],"epsgaxis=") != NULL )
+        {
+            axis = strstr(proj->args[i],"=") + 1;
+            break;
+        }
+    }
+
+    if( axis == NULL )
+        return;
+
+    if( strcasecmp(axis,"en") == 0 )
+        return;
+
+    if( strcasecmp(axis,"ne") != 0 )
+    {
+        msDebug( "msAxisNormalizePoints(): odd +epsgaxis= value: '%s'.",
+                 axis );
+        return;
+    }
+
+    /* Switch axes */
+    for( i = 0; i < count; i++ )
+    {
+        double tmp;
+
+        tmp = x[i];
+        x[i] = y[i];
+        y[i] = tmp;
+    }
+}
+
+/************************************************************************/
+/*                      msAxisDenormalizePoints()                       */
+/*                                                                      */
+/*      Convert points from easting,northing orientation to the         */
+/*      preferred epsg orientation of this projectionObj.               */
+/************************************************************************/
+
+void msAxisDenormalizePoints( projectionObj *proj, int count, 
+                              double *x, double *y )
+
+{
+    /* For how this is essentially identical to normalizing */
+    msAxisNormalizePoints( proj, count, x, y );
+}
