@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapquery.c 9561 2009-11-25 14:50:54Z sdlime $
+ * $Id: mapquery.c 10113 2010-04-24 17:07:20Z sdlime $
  *
  * Project:  MapServer
  * Purpose:  layer query support.
@@ -29,7 +29,7 @@
 
 #include "mapserver.h"
 
-MS_CVSID("$Id: mapquery.c 9561 2009-11-25 14:50:54Z sdlime $")
+MS_CVSID("$Id: mapquery.c 10113 2010-04-24 17:07:20Z sdlime $")
 
 int msInitQuery(queryObj *query)
 {
@@ -220,7 +220,10 @@ int msLoadQuery(mapObj *map, char *filename) {
   /* 
   ** Make sure the file at least has the right extension. 
   */ 
-  if(msEvalRegex("\\.qy$", filename) != MS_TRUE) return MS_FAILURE; 
+  if(msEvalRegex("\\.qy$", filename) != MS_TRUE) {
+    msSetError(MS_MISCERR, "Query file extension check failed on %s (extension must be .qy)", "msLoadQuery()", filename);
+    return MS_FAILURE; 
+  }
 
   stream = fopen(filename, "r");
   if(!stream) {
@@ -362,9 +365,7 @@ int msQueryByIndex(mapObj *map)
 
   if(map->query.clear_resultcache || lp->resultcache == NULL) {
     lp->resultcache = (resultCacheObj *)malloc(sizeof(resultCacheObj)); /* allocate and initialize the result cache */
-    lp->resultcache->results = NULL;
-    lp->resultcache->numresults = lp->resultcache->cachesize = 0;
-    lp->resultcache->bounds.minx = lp->resultcache->bounds.miny = lp->resultcache->bounds.maxx = lp->resultcache->bounds.maxy = -1;
+    initResultCache( lp->resultcache);
   }
 
   msInitShape(&shape);
@@ -500,7 +501,7 @@ int msQueryByAttributes(mapObj *map)
   }
 
   /* identify candidate shapes */
-  searchrect = map->extent;
+  searchrect = map->query.rect;
 #ifdef USE_PROJ  
   if(lp->project && msProjectionsDiffer(&(lp->projection), &(map->projection)))  
     msProjectRect(&(map->projection), &(lp->projection), &searchrect); /* project the searchrect to source coords */
@@ -521,9 +522,7 @@ int msQueryByAttributes(mapObj *map)
   }
 
   lp->resultcache = (resultCacheObj *)malloc(sizeof(resultCacheObj)); /* allocate and initialize the result cache */
-  lp->resultcache->results = NULL;
-  lp->resultcache->numresults = lp->resultcache->cachesize = 0;
-  lp->resultcache->bounds.minx = lp->resultcache->bounds.miny = lp->resultcache->bounds.maxx = lp->resultcache->bounds.maxy = -1;
+  initResultCache( lp->resultcache);
   
   nclasses = 0;
   classgroup = NULL;
@@ -674,9 +673,7 @@ int msQueryByRect(mapObj *map)
     }
 
     lp->resultcache = (resultCacheObj *)malloc(sizeof(resultCacheObj)); /* allocate and initialize the result cache */
-    lp->resultcache->results = NULL;
-    lp->resultcache->numresults = lp->resultcache->cachesize = 0;
-    lp->resultcache->bounds.minx = lp->resultcache->bounds.miny = lp->resultcache->bounds.maxx = lp->resultcache->bounds.maxy = -1;
+    initResultCache( lp->resultcache);
 
     nclasses = 0;
     classgroup = NULL;
@@ -908,9 +905,7 @@ int msQueryByFeatures(mapObj *map)
 
       if(i == 0) {
 	lp->resultcache = (resultCacheObj *)malloc(sizeof(resultCacheObj)); /* allocate and initialize the result cache */
-	lp->resultcache->results = NULL;
-	lp->resultcache->numresults = lp->resultcache->cachesize = 0;
-	lp->resultcache->bounds.minx = lp->resultcache->bounds.miny = lp->resultcache->bounds.maxx = lp->resultcache->bounds.maxy = -1;    
+        initResultCache( lp->resultcache);
       }      
 
       nclasses = 0;
@@ -1166,9 +1161,7 @@ int msQueryByPoint(mapObj *map)
     }
 
     lp->resultcache = (resultCacheObj *)malloc(sizeof(resultCacheObj)); /* allocate and initialize the result cache */
-    lp->resultcache->results = NULL;
-    lp->resultcache->numresults = lp->resultcache->cachesize = 0;
-    lp->resultcache->bounds.minx = lp->resultcache->bounds.miny = lp->resultcache->bounds.maxx = lp->resultcache->bounds.maxy = -1;
+    initResultCache( lp->resultcache);
 
     nclasses = 0;
     classgroup = NULL;
@@ -1357,9 +1350,7 @@ int msQueryByShape(mapObj *map)
     }
 
     lp->resultcache = (resultCacheObj *)malloc(sizeof(resultCacheObj)); /* allocate and initialize the result cache */
-    lp->resultcache->results = NULL;
-    lp->resultcache->numresults = lp->resultcache->cachesize = 0;
-    lp->resultcache->bounds.minx = lp->resultcache->bounds.miny = lp->resultcache->bounds.maxx = lp->resultcache->bounds.maxy = -1;
+    initResultCache( lp->resultcache);
 
     nclasses = 0;
     classgroup = NULL;
@@ -1528,11 +1519,8 @@ int msQueryByOperator(mapObj *map)
     if(status != MS_SUCCESS) return(MS_FAILURE);
 
     /* identify target shapes */
-    searchrect.minx = map->extent.minx;
-    searchrect.miny = map->extent.miny;
-    searchrect.maxx = map->extent.maxx;
-    searchrect.maxy = map->extent.maxy;
-      
+    searchrect = qshape->bounds;
+
 #ifdef USE_PROJ
     if(lp->project && msProjectionsDiffer(&(lp->projection), &(map->projection)))
       msProjectRect(&(map->projection), &(lp->projection), &searchrect); /* project the searchrect to source coords */
@@ -1549,9 +1537,7 @@ int msQueryByOperator(mapObj *map)
     }
 
     lp->resultcache = (resultCacheObj *)malloc(sizeof(resultCacheObj)); /* allocate and initialize the result cache */
-    lp->resultcache->results = NULL;
-    lp->resultcache->numresults = lp->resultcache->cachesize = 0;
-    lp->resultcache->bounds.minx = lp->resultcache->bounds.miny = lp->resultcache->bounds.maxx = lp->resultcache->bounds.maxy = -1;
+    initResultCache( lp->resultcache);
 
     classgroup = NULL;
     if (lp->classgroup && lp->numclasses > 0)
