@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapraster.c 9440 2009-10-11 21:27:28Z warmerdam $
+ * $Id: mapraster.c 10093 2010-04-10 15:26:28Z warmerdam $
  *
  * Project:  MapServer
  * Purpose:  msDrawRasterLayer(): generic raster layer drawing, including
@@ -36,7 +36,7 @@
 #include "mapresample.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapraster.c 9440 2009-10-11 21:27:28Z warmerdam $")
+MS_CVSID("$Id: mapraster.c 10093 2010-04-10 15:26:28Z warmerdam $")
 
 extern int msyyparse(void);
 extern int msyylex(void);
@@ -171,7 +171,12 @@ int msGetClass_Float(layerObj *layer, float fValue)
         switch(layer->class[i]->expression.type) {
           case(MS_STRING):
             sprintf(tmpstr2, "%18g", fValue );
-            if(strcmp(layer->class[i]->expression.string, tmpstr2) == 0) return(i); /* matched */
+            /* trim junk white space */
+            tmpstr1= tmpstr2;
+            while( *tmpstr1 == ' ' )
+                tmpstr1++;
+
+            if(strcmp(layer->class[i]->expression.string, tmpstr1) == 0) return(i); /* matched */
             break;
 
           case(MS_REGEX):
@@ -1533,9 +1538,7 @@ int msDrawRasterLayerLow(mapObj *map, layerObj *layer, imageObj *image)
                 }
             }
 
-            msReleaseLock( TLOCK_GDAL );
             msGetGDALGeoTransform( hDS, map, layer, adfGeoTransform );
-            msAcquireLock( TLOCK_GDAL );
 
             /* 
             ** We want to resample if the source image is rotated, if
