@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: maprasterquery.c 9502 2009-10-22 20:37:44Z warmerdam $
+ * $Id: maprasterquery.c 10001 2010-03-24 16:30:22Z warmerdam $
  *
  * Project:  MapServer
  * Purpose:  Implementation of query operations on rasters. 
@@ -32,7 +32,7 @@
 #include "mapresample.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: maprasterquery.c 9502 2009-10-22 20:37:44Z warmerdam $")
+MS_CVSID("$Id: maprasterquery.c 10001 2010-03-24 16:30:22Z warmerdam $")
 
 int msRASTERLayerGetShape(layerObj *layer, shapeObj *shape, int tile, long record);
 int msRASTERLayerGetItems(layerObj *layer);
@@ -1328,6 +1328,7 @@ int msRASTERLayerGetExtent(layerObj *layer, rectObj *extent)
   double adfGeoTransform[6];
   int nXSize, nYSize;
   GDALDatasetH hDS;
+  CPLErr eErr = CE_Failure;
 
   /*
   ** For the time being we only automatically derive extents from
@@ -1358,17 +1359,25 @@ int msRASTERLayerGetExtent(layerObj *layer, rectObj *extent)
   {
     nXSize = GDALGetRasterXSize( hDS );
     nYSize = GDALGetRasterYSize( hDS );
-    GDALGetGeoTransform( hDS, adfGeoTransform );
+    eErr = GDALGetGeoTransform( hDS, adfGeoTransform );
     
     GDALClose( hDS );
   }
   
   msReleaseLock( TLOCK_GDAL );
 
-  if( hDS == NULL )
+  if( hDS == NULL || eErr != CE_None )
   {
       return MS_FAILURE;
   }
+
+  /* If this appears to be an ungeoreferenced raster than flip it for
+     mapservers purposes. */
+  if( adfGeoTransform[5] == 1.0 && adfGeoTransform[3] == 0.0 )
+  {
+      adfGeoTransform[5] = -1.0;
+      adfGeoTransform[3] = nYSize;
+  }
   
   extent->minx = adfGeoTransform[0];
   extent->maxy = adfGeoTransform[3];
