@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: AssemblyInfo.cs 7418 2008-02-29 00:02:49Z nsavard $
+ * $Id: AssemblyInfo.cs 10131 2010-05-02 12:34:21Z tamas $
  *
  * Project:  MapServer
  * Purpose:  MapScript CSharp Assembly Information
@@ -92,3 +92,7 @@ using System.Security;
 // Web page that should be running under restricted permissions.
 [assembly: AllowPartiallyTrustedCallers]
 
+// Use the .NET Framework 2.0 transparency rules (level 1 transparency) as default
+#if (CLR4)
+[assembly: SecurityRules(SecurityRuleSet.Level1)]
+#endif
\ No newline at end of file
