@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: php_mapscript.h 8426 2009-01-12 16:57:27Z aboudreault $
+ * $Id: php_mapscript.h 9558 2009-11-20 18:11:50Z sdlime $
  *
  * Project:  MapServer
  * Purpose:  PHP/MapScript extension for MapServer. Header file 
@@ -130,6 +130,11 @@ int             mapObj_OWSDispatch(mapObj *self, cgiRequestObj *req );
 int             mapObj_insertLayer(mapObj *self, layerObj *layer, int index);
 layerObj        *mapObj_removeLayer(mapObj *self, int layerindex);
 
+int             mapObj_setCenter(mapObj *self, pointObj *center);
+int             mapObj_offsetExtent(mapObj *self, double x, double y);
+int             mapObj_scaleExtent(mapObj *self, double zoomfactor, double minscaledenom, 
+                                   double maxscaledenom);
+
 layerObj       *layerObj_new(mapObj *map);
 void            layerObj_destroy(layerObj* self);
 int             layerObj_updateFromString(layerObj *self, char *snippet);
@@ -256,6 +261,8 @@ pointObj        *shapeObj_getpointusingmeasure(shapeObj *self, double m);
 pointObj        *shapeObj_getmeasureusingpoint(shapeObj *self, pointObj *point);
 
 shapeObj        *shapeObj_buffer(shapeObj *self, double width);
+shapeObj        *shapeObj_simplify(shapeObj *self, double tolerance);
+shapeObj        *shapeObj_topologypreservingsimplify(shapeObj *self, double tolerance);
 shapeObj        *shapeObj_convexHull(shapeObj *self);
 shapeObj        *shapeObj_boundary(shapeObj *self);
 int             shapeObj_contains_geos(shapeObj *self, shapeObj *poshape);
