@@ -1,11 +1,9 @@
 /**********************************************************************
- * $Id: php_mapscript_util.c 6214 2007-06-15 14:10:05Z Assefa $
+ * $Id: php_mapscript_util.c 8445 2009-01-21 16:32:09Z aboudreault $
  *
- * Name:     php_mapscript_util.c
- * Project:  PHP/MapScript extension for MapServer : Utility functions
- * Language: ANSI C
- * Purpose:  Utility functions
- * Author:   Daniel Morissette, morissette@dmsolutions.ca
+ * Project:  MapServer
+ * Purpose:  PHP/MapScript extension for MapServer : Utility functions
+ * Author:   Daniel Morissette, DM Solutions Group (dmorissette@dmsolutions.ca)
  *
  **********************************************************************
  * Copyright (c) 2000-2005, DM Solutions Group
@@ -411,7 +409,10 @@ int _phpms_add_property_object(pval *pObj,
           php3_error(err_type, "Unable to add %s property", property_name);
         return -1;
     }
-
+#if PHP_MAJOR_VERSION >= 5
+    else
+        ZVAL_DELREF(pObjToAdd);
+#endif
     return 0;
 }
 
