@@ -1,4 +1,4 @@
-# $Id: imagetest.py 4353 2005-01-30 02:18:27Z sean $
+# $Id: imagetest.py 6192 2007-06-11 01:40:25Z hobu $
 #
 # Project:  MapServer
 # Purpose:  xUnit style Python mapscript tests of map cloning
@@ -128,11 +128,11 @@ class ImageObjTestCase(unittest.TestCase):
     
     def testConstructorUrlStream(self):
         """imageObj with a URL stream works"""
-        url = urllib.urlopen('http://mapserver.gis.umn.edu/bugs/ant.jpg')
+        url = urllib.urlopen('http://mapserver.gis.umn.edu/logo.jpg')
         imgobj = mapscript.imageObj(url, 'GD/JPEG')
         assert imgobj.thisown == 1
-        assert imgobj.height == 220
-        assert imgobj.width == 329
+        assert imgobj.height == 68
+        assert imgobj.width == 439
         imgobj.save('testConstructorUrlStream.jpg')
 
 class ImageWriteTestCase(MapTestCase):
