@@ -1,4 +1,4 @@
-# $Id: layertest.py 4290 2005-01-07 19:13:08Z sean $
+# $Id: layertest.py 6519 2007-08-08 20:37:34Z hobu $
 #
 # Project:  MapServer
 # Purpose:  xUnit style Python mapscript tests of Layer
@@ -65,6 +65,7 @@ class LayerConstructorTestCase(MapLayerTestCase):
         assert str(t) == "<class 'mapscript.layerObj'>", t
         assert layer.thisown == 1
         assert layer.index == -1
+	assert layer.map == None, layer.map
     
     def testLayerConstructorMapArg(self):
         """test layer constructor with map argument"""
@@ -73,6 +74,8 @@ class LayerConstructorTestCase(MapLayerTestCase):
         assert str(t) == "<class 'mapscript.layerObj'>", t
         assert layer.thisown == 1
         assert str(layer) == str(self.map.getLayer(self.map.numlayers-1))
+	assert layer.map != None, layer.map
+    
      
 
 class LayerCloningTestCase(MapLayerTestCase):
@@ -183,7 +186,7 @@ class InsertClassTestCase(MapLayerTestCase):
         assert new_index == n
         assert self.layer.numclasses == n + 1
         c = self.layer.getClass(new_index)
-        assert c.thisown == 0
+        assert c.thisown == 1 
         assert c.name == new_class.name
         
     def testLayerInsertClassAtZero(self):
@@ -195,7 +198,7 @@ class InsertClassTestCase(MapLayerTestCase):
         assert new_index == 0
         assert self.layer.numclasses == n + 1
         c = self.layer.getClass(new_index)
-        assert c.thisown == 0
+        assert c.thisown == 1 
         assert c.name == new_class.name
 
     def testInsertNULLClass(self):
@@ -250,31 +253,48 @@ class ExpressionTestCase(MapLayerTestCase):
         """layer expression can be properly cleared"""
         self.layer.setFilter('')
         fs = self.layer.getFilterString()
-        assert fs == '"(null)"', fs
+        assert fs == None, fs
     
     def testSetStringExpression(self):
         """layer expression can be set to string"""
         self.layer.setFilter('foo')
         fs = self.layer.getFilterString()
+        self.layer.filteritem = 'fid'
         assert fs == '"foo"', fs
+        self.map.draw()
     
     def testSetQuotedStringExpression(self):
         """layer expression string can be quoted"""
         self.layer.setFilter('"foo"')
         fs = self.layer.getFilterString()
+        self.layer.filteritem = 'fid'
         assert fs == '"foo"', fs
+        self.map.draw()
     
     def testSetRegularExpression(self):
         """layer expression can be regular expression"""
         self.layer.setFilter('/foo/')
+        self.layer.filteritem = 'fid'
         fs = self.layer.getFilterString()
         assert fs == '/foo/', fs
+        self.map.draw()
     
     def testSetLogicalExpression(self):
         """layer expression can be logical expression"""
-        self.layer.setFilter('([foo] >= 2)')
+        self.layer.setFilter('([fid] >= 2)')
         fs = self.layer.getFilterString()
-        assert fs == '([foo] >= 2)', fs
+        assert fs == '([fid] >= 2)', fs
+        self.map.draw()
+       
+    def testSetCompoundLogicalExpression(self):
+        """layer expression can be a compound logical expression"""
+	#filter = '( ([fid] >= 2) AND (\'[fname]\' == \'A Polygon\' ))'
+        flt = """( ([fid] >= 2) AND ("[fname]" == 'A Polygon' ))"""
+        excode = self.layer.setFilter(flt)
+        assert excode == mapscript.MS_SUCCESS, excode
+        fs = self.layer.getFilterString()
+        assert fs == flt, fs
+        self.map.draw()
        
 
 class LayerQueryTestCase(MapLayerTestCase):
@@ -341,8 +361,8 @@ class LayerVisibilityTestCase(MapLayerTestCase):
     
     def setUp(self):
         MapLayerTestCase.setUp(self)
-        self.layer.minscale = 1000
-        self.layer.maxscale = 2000
+        self.layer.minscaledenom = 1000
+        self.layer.maxscaledenom = 2000
         self.layer.status = mapscript.MS_ON
         self.map.zoomScale(1500, mapscript.pointObj(100,100), 200, 200, self.map.extent, None)
     
