@@ -1,4 +1,4 @@
-# $Id: symboltest.py 4171 2004-11-27 16:39:33Z sean $
+# $Id: symboltest.py 6522 2007-08-08 20:43:38Z hobu $
 #
 # Project:  MapServer
 # Purpose:  xUnit style Python mapscript tests of Symbol
@@ -150,7 +150,7 @@ class MapSymbolTestCase(MapTestCase):
     def testSetStyle(self):
         """expect success after setting an existing symbol's style"""
         symbol = self.map.symbolset.getSymbol(1)
-        assert symbol.setStyle(0, 1) == mapscript.MS_SUCCESS
+        assert symbol.setPattern(0, 1) == mapscript.MS_SUCCESS
 
     def testRGBASymbolInPNG24(self):
         """draw a RGBA PNG pixmap on PNG canvas"""
