@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapsde.c 9200 2009-07-22 04:20:44Z sdlime $
+ * $Id: mapsde.c 9956 2010-03-19 18:25:33Z pramsey $
  *    
  * Project:  MapServer
  * Purpose:  Implements SDE CONNECTIONTYPE.
@@ -762,6 +762,13 @@ static int sdeGetRecord(layerObj *layer, shapeObj *shape) {
                     else
                         shape->values[i] = msConvertWideStringToUTF8((const wchar_t*) wide, "UTF-16LE");
                     msFree(wide);
+                    if (!shape->values[i]) {  /* There was an error */
+                        msSetError( MS_SDEERR,
+                                 "msConvertWideStringToUTF8()==NULL.",
+                                 "sdeGetRecord()");
+                        shape->values[i] = (char *)malloc(itemdefs[i].size*sizeof(char)+1);
+                        shape->values[i][0] = '\0'; /* empty string */
+                    }                    
                 }
                 break;
 #endif
