@@ -39,7 +39,7 @@
 #include <signal.h>
 #endif
 
-MS_CVSID("$Id: mapserv.c 9649 2010-01-02 17:21:10Z sdlime $")
+MS_CVSID("$Id: mapserv.c 10306 2010-07-08 20:07:15Z dmorissette $")
 
 mapservObj* mapserv;
 
@@ -1139,13 +1139,23 @@ int main(int argc, char *argv[]) {
   /*      purposes, and to query the version info.                        */
   /* -------------------------------------------------------------------- */
   for( iArg = 1; iArg < argc; iArg++ ) {
+    /* Keep only "-v", "-nh" and "QUERY_STRING=..." enabled by default.
+     * The others will require an explicit -DMS_ENABLE_CGI_CL_DEBUG_ARGS
+     * at compile time.
+     */
     if( strcmp(argv[iArg],"-v") == 0 ) {
       printf("%s\n", msGetVersion());
       fflush(stdout);
       exit(0);
     } else if(strcmp(argv[iArg], "-nh") == 0) {
       sendheaders = MS_FALSE;
-    } else if( iArg < argc-1 && strcmp(argv[iArg], "-tmpbase") == 0) {
+    } else if( strncmp(argv[iArg], "QUERY_STRING=", 13) == 0 ) {
+      /* Debugging hook... pass "QUERY_STRING=..." on the command-line */
+      putenv( "REQUEST_METHOD=GET" );
+      putenv( argv[iArg] );
+    }
+#ifdef MS_ENABLE_CGI_CL_DEBUG_ARGS
+    else if( iArg < argc-1 && strcmp(argv[iArg], "-tmpbase") == 0) {
       msForceTmpFileBase( argv[++iArg] );
     } else if( iArg < argc-1 && strcmp(argv[iArg], "-t") == 0) {
       char **tokens;
@@ -1161,15 +1171,13 @@ int main(int argc, char *argv[]) {
       }
             
       exit(0);
-    } else if( strncmp(argv[iArg], "QUERY_STRING=", 13) == 0) {
-      /* Debugging hook... pass "QUERY_STRING=..." on the command-line */
-      putenv( "REQUEST_METHOD=GET" );
-      putenv( argv[iArg] );
     } else if( strncmp(argv[iArg], "MS_ERRORFILE=", 13) == 0 ) {
       msSetErrorFile( argv[iArg] + 13 );
     } else if( strncmp(argv[iArg], "MS_DEBUGLEVEL=", 14) == 0) {
       msSetGlobalDebugLevel( atoi(argv[iArg] + 14) );
-    } else {
+    }
+#endif /* MS_ENABLE_CGI_CL_DEBUG_ARGS */
+    else {
       /* we don't produce a usage message as some web servers pass junk arguments */
     }
   }
@@ -1562,6 +1570,8 @@ int main(int argc, char *argv[]) {
           if(QueryItem) mapserv->map->query.item = strdup(QueryItem);
           if(QueryString) mapserv->map->query.str = strdup(QueryString);
 
+          mapserv->map->query.rect = mapserv->map->extent;
+
           mapserv->map->query.mode = MS_QUERY_MULTIPLE;
           if(mapserv->Mode == ITEMFEATUREQUERY || mapserv->Mode == ITEMFEATUREQUERYMAP)
             mapserv->map->query.mode = MS_QUERY_SINGLE;
@@ -1654,6 +1664,8 @@ int main(int argc, char *argv[]) {
           if(QueryItem) mapserv->map->query.item = strdup(QueryItem);
           if(QueryString) mapserv->map->query.str = strdup(QueryString);
 
+          mapserv->map->query.rect = mapserv->map->extent;
+
 	  mapserv->map->query.mode = MS_QUERY_MULTIPLE;
           if(mapserv->Mode == ITEMQUERY || mapserv->Mode == ITEMQUERYMAP) mapserv->map->query.mode = MS_QUERY_SINGLE;
           break;
