@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapshape.c 8015 2008-11-11 18:14:14Z pramsey $
+ * $Id: mapshape.c 9509 2009-10-24 04:45:32Z sdlime $
  *
  * Project:  MapServer
  * Purpose:  Implements support for shapefile access.
@@ -38,7 +38,7 @@
 #include <assert.h>
 #include "mapserver.h"
 
-MS_CVSID("$Id: mapshape.c 8015 2008-11-11 18:14:14Z pramsey $")
+MS_CVSID("$Id: mapshape.c 9509 2009-10-24 04:45:32Z sdlime $")
 
 /* Only use this macro on 32-bit integers! */
 #define SWAP_FOUR_BYTES(data) \
@@ -1756,9 +1756,7 @@ int msShapefileWhichShapes(shapefileObj *shpfile, rectObj rect, int debug)
       msSetError(MS_MEMERR, NULL, "msShapefileWhichShapes()");
       return(MS_FAILURE);
     }
-    for(i=0;i<shpfile->numshapes;i++) {
-      msSetBit(shpfile->status, i, 1);
-    }
+    msSetAllBits(shpfile->status, shpfile->numshapes, 1);
   } 
   else {
 
@@ -1895,8 +1893,8 @@ int msTiledSHPOpenFile(layerObj *layer)
     status = msLayerOpen(tlp);
     if(status != MS_SUCCESS) return(MS_FAILURE);
 
-     /* build item list (no annotation) since we do have to classify the shape */
-     status = msLayerWhichItems(tlp, MS_TRUE, MS_FALSE, NULL);
+     /* build item list */
+     status = msLayerWhichItems(tlp, MS_FALSE, NULL);
      if(status != MS_SUCCESS) return(MS_FAILURE);
 
      tSHP->tileshpfile = (shapefileObj *) tlp->layerinfo; /* shapefiles use layerinfo to point to a shapefileObj */
@@ -2294,8 +2292,16 @@ void msTiledSHPClose(layerObj *layer)
 /************************************************************************/
 int msTiledSHPCloseVT(layerObj *layer) 
 {
-	msTiledSHPClose(layer);
-	return MS_SUCCESS;
+  msTiledSHPClose(layer);
+  return MS_SUCCESS;
+}
+
+void msTiledSHPLayerFreeItemInfo(layerObj *layer)
+{
+  if(layer->iteminfo) {
+    free(layer->iteminfo);
+    layer->iteminfo = NULL;
+  }
 }
 
 int msTiledSHPLayerInitItemInfo(layerObj *layer)
@@ -2308,6 +2314,7 @@ int msTiledSHPLayerInitItemInfo(layerObj *layer)
     return MS_FAILURE;
   }
 
+  msTiledSHPLayerFreeItemInfo(layer);
   layer->iteminfo = (int *) msDBFGetItemIndexes(tSHP->shpfile->hDBF, layer->items, layer->numitems);
   if(!layer->iteminfo) return(MS_FAILURE);
 
@@ -2345,54 +2352,50 @@ int msTiledSHPLayerGetExtent(layerObj *layer, rectObj *extent)
   return MS_SUCCESS;
 }
 
-void msTiledSHPLayerFreeItemInfo(layerObj *layer)
-{
-	if(layer->iteminfo) {
-		free(layer->iteminfo);
-		layer->iteminfo = NULL;
-	}
-}
-
 int msTiledSHPLayerIsOpen(layerObj *layer)
 {
-	if(layer->layerinfo)
-		return MS_TRUE;
-	else
-		return MS_FALSE;
+  if(layer->layerinfo)
+    return MS_TRUE;
+  else
+    return MS_FALSE;
 }
 
 int msTiledSHPLayerInitializeVirtualTable(layerObj *layer)
 {
-	assert(layer != NULL);
-	assert(layer->vtable != NULL);
-
-	layer->vtable->LayerInitItemInfo = msTiledSHPLayerInitItemInfo;
-	layer->vtable->LayerFreeItemInfo = msTiledSHPLayerFreeItemInfo;
-	layer->vtable->LayerOpen = msTiledSHPOpenFile;
-
-	layer->vtable->LayerIsOpen = msTiledSHPLayerIsOpen;
-	layer->vtable->LayerWhichShapes = msTiledSHPWhichShapes;
-	layer->vtable->LayerNextShape = msTiledSHPNextShape;
-	layer->vtable->LayerGetShape = msTiledSHPGetShape;
-
-	layer->vtable->LayerClose = msTiledSHPCloseVT;
-	layer->vtable->LayerGetItems = msTiledSHPLayerGetItems;
-	layer->vtable->LayerGetExtent = msTiledSHPLayerGetExtent;
+  assert(layer != NULL);
+  assert(layer->vtable != NULL);
 
+  layer->vtable->LayerInitItemInfo = msTiledSHPLayerInitItemInfo;
+  layer->vtable->LayerFreeItemInfo = msTiledSHPLayerFreeItemInfo;
+  layer->vtable->LayerOpen = msTiledSHPOpenFile;
+  layer->vtable->LayerIsOpen = msTiledSHPLayerIsOpen;
+  layer->vtable->LayerWhichShapes = msTiledSHPWhichShapes;
+  layer->vtable->LayerNextShape = msTiledSHPNextShape;
+  layer->vtable->LayerResultsGetShape = msTiledSHPGetShape; /* no special version, use ...GetShape() */
+  layer->vtable->LayerGetShape = msTiledSHPGetShape;
+  layer->vtable->LayerClose = msTiledSHPCloseVT;
+  layer->vtable->LayerGetItems = msTiledSHPLayerGetItems;
+  layer->vtable->LayerGetExtent = msTiledSHPLayerGetExtent;
   /* layer->vtable->LayerApplyFilterToLayer, use default */
-
-	/* layer->vtable->LayerGetAutoStyle, use default */
-	/* layer->vtable->LayerCloseConnection, use default */;
-
-	layer->vtable->LayerSetTimeFilter = msLayerMakeBackticsTimeFilter;
-	/* layer->vtable->LayerCreateItems, use default */
+  /* layer->vtable->LayerGetAutoStyle, use default */
+  /* layer->vtable->LayerCloseConnection, use default */;
+  layer->vtable->LayerSetTimeFilter = msLayerMakeBackticsTimeFilter;
+  /* layer->vtable->LayerCreateItems, use default */
   /* layer->vtable->LayerGetNumFeatures, use default */
 
-	return MS_SUCCESS;
+  return MS_SUCCESS;
 }
 
 /* SHAPEFILE Layer virtual table functions */
 
+void msShapeFileLayerFreeItemInfo(layerObj *layer) 
+{ 
+  if(layer->iteminfo) {
+    free(layer->iteminfo);
+    layer->iteminfo = NULL;
+  }
+}
+
 int msShapeFileLayerInitItemInfo(layerObj *layer) 
 {
   shapefileObj *shpfile = shpfile = layer->layerinfo;
@@ -2402,6 +2405,7 @@ int msShapeFileLayerInitItemInfo(layerObj *layer)
   }
 
   /* iteminfo needs to be a bit more complex, a list of indexes plus the length of the list */
+  msShapeFileLayerFreeItemInfo(layer);
   layer->iteminfo = (int *) msDBFGetItemIndexes(shpfile->hDBF, layer->items, layer->numitems);
   if( ! layer->iteminfo) {
     return MS_FAILURE;
@@ -2410,16 +2414,6 @@ int msShapeFileLayerInitItemInfo(layerObj *layer)
   return MS_SUCCESS;
 }
 
-
-void msShapeFileLayerFreeItemInfo(layerObj *layer) 
-{ 
-  if(layer->iteminfo) {
-    free(layer->iteminfo);
-    layer->iteminfo = NULL;
-  }
-}
-
-
 int msShapeFileLayerOpen(layerObj *layer)
 {
   char szPath[MS_MAXPATHLEN];
@@ -2584,7 +2578,8 @@ int msShapeFileLayerGetItems(layerObj *layer)
   }
 
   layer->numitems = msDBFGetFieldCount(shpfile->hDBF);
-  layer->items = msDBFGetItems(shpfile->hDBF);    
+  layer->items = msDBFGetItems(shpfile->hDBF);
+  if(layer->numitems == 0) return MS_SUCCESS; /* No items is a valid case (#3147) */
   if(!layer->items) return MS_FAILURE;
 
   return msLayerInitItemInfo(layer);
@@ -2607,6 +2602,7 @@ int msShapeFileLayerInitializeVirtualTable(layerObj *layer)
   layer->vtable->LayerIsOpen = msShapeFileLayerIsOpen;
   layer->vtable->LayerWhichShapes = msShapeFileLayerWhichShapes;
   layer->vtable->LayerNextShape = msShapeFileLayerNextShape;
+  layer->vtable->LayerResultsGetShape = msShapeFileLayerGetShape; /* no special version, use ...GetShape() */
   layer->vtable->LayerGetShape = msShapeFileLayerGetShape;
   layer->vtable->LayerClose = msShapeFileLayerClose;
   layer->vtable->LayerGetItems = msShapeFileLayerGetItems;
