@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: maptree.h 6428 2007-07-31 19:58:45Z dmorissette $
+ * $Id: maptree.h 8741 2009-03-09 21:04:52Z jlacroix $
  *
  * Project:  MapServer
  * Purpose:  .qix spatial index declarations.
@@ -85,13 +85,13 @@ MS_DLL_EXPORT treeObj *msCreateTree(shapefileObj *shapefile, int maxdepth);
 MS_DLL_EXPORT void msTreeTrim(treeObj *tree);
 MS_DLL_EXPORT void msDestroyTree(treeObj *tree);
 
-MS_DLL_EXPORT char *msSearchTree(treeObj *tree, rectObj aoi);
-MS_DLL_EXPORT char *msSearchDiskTree(char *filename, rectObj aoi, int debug);
+MS_DLL_EXPORT ms_bitarray msSearchTree(treeObj *tree, rectObj aoi);
+MS_DLL_EXPORT ms_bitarray msSearchDiskTree(char *filename, rectObj aoi, int debug);
 
 MS_DLL_EXPORT treeObj *msReadTree(char *filename, int debug);
 MS_DLL_EXPORT int msWriteTree(treeObj *tree, char *filename, int LSB_order);
 
-MS_DLL_EXPORT void msFilterTreeSearch(shapefileObj *shp, char *status, rectObj search_rect);
+MS_DLL_EXPORT void msFilterTreeSearch(shapefileObj *shp, ms_bitarray status, rectObj search_rect);
 
 #ifdef __cplusplus
 }
