@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: maputil.c 9472 2009-10-16 18:17:05Z aboudreault $
+ * $Id: maputil.c 10305 2010-07-08 20:04:54Z dmorissette $
  *
  * Project:  MapServer
  * Purpose:  Various utility functions ... a real hodgepodge.
@@ -50,7 +50,7 @@
 #include <process.h>
 #endif
 
-MS_CVSID("$Id: maputil.c 9472 2009-10-16 18:17:05Z aboudreault $")
+MS_CVSID("$Id: maputil.c 10305 2010-07-08 20:04:54Z dmorissette $")
 
 extern int msyyparse(void);
 extern int msyylex(void);
@@ -1347,22 +1347,24 @@ char *msTmpFile(const char *mappath, const char *tmppath, const char *ext)
     char szPath[MS_MAXPATHLEN];
     const char *fullFname;
     char tmpId[128]; /* big enough for time + pid + ext */
+    const char *tmpBase = NULL;
 
     if( ForcedTmpBase != NULL )
     {
-        strncpy( tmpId, ForcedTmpBase, sizeof(tmpId) );
+        tmpBase = ForcedTmpBase;
     }
     else 
     {
         /* We'll use tmpId and tmpCount to generate unique filenames */
         sprintf(tmpId, "%lx_%x",(long)time(NULL),(int)getpid());
+        tmpBase = tmpId;
     }
 
     if (ext == NULL)  ext = "";
-    tmpFname = (char*)malloc(strlen(tmpId) + 10  + strlen(ext) + 1);
+    tmpFname = (char*)malloc(strlen(tmpBase) + 10  + strlen(ext) + 1);
 
     msAcquireLock( TLOCK_TMPFILE );
-    sprintf(tmpFname, "%s_%x.%s", tmpId, tmpCount++, ext);
+    sprintf(tmpFname, "%s_%x.%s", tmpBase, tmpCount++, ext);
     msReleaseLock( TLOCK_TMPFILE );
 
     fullFname = msBuildPath3(szPath, mappath, tmppath, tmpFname);
