@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mapwfs11.c 7804 2008-07-09 11:27:53Z tomkralidis $
+ * $Id: mapwfs11.c 9498 2009-10-20 13:38:46Z aboudreault $
  *
  * Project:  MapServer
  * Purpose:  OGC WFS 1.1.0 implementation. This file holds some WFS 1.1.0 
@@ -30,7 +30,7 @@
 #include "mapserver.h"
 #include "mapows.h"
 
-MS_CVSID("$Id: mapwfs11.c 7804 2008-07-09 11:27:53Z tomkralidis $")
+MS_CVSID("$Id: mapwfs11.c 9498 2009-10-20 13:38:46Z aboudreault $")
 
 #if defined(USE_WFS_SVR) && defined(USE_LIBXML2)
 #include "maplibxml2.h"
@@ -45,18 +45,19 @@ int msWFSException11(mapObj *map, const char *locator,
     char *errorString     = NULL;
     char *errorMessage    = NULL;
     char *schemasLocation = NULL;
+    const char *encoding;
 
     xmlDocPtr  psDoc      = NULL;   
     xmlNodePtr psRootNode = NULL;
     xmlNsPtr   psNsOws    = NULL;
     xmlChar *buffer       = NULL;
     
-
     if (version == NULL)
        version = "1.1.0";
 
+    psNsOws = xmlNewNs(NULL, BAD_CAST "http://www.opengis.net/ows", BAD_CAST "ows");
 
-    psNsOws = xmlNewNs(NULL, BAD_CAST "http://www.opengis.net/ows/1.1", BAD_CAST "ows");
+    encoding = msOWSLookupMetadata(&(map->web.metadata), "FO", "encoding");
 
     errorString = msGetErrorString("\n");
     errorMessage = msEncodeHTMLEntities(errorString);
@@ -64,14 +65,18 @@ int msWFSException11(mapObj *map, const char *locator,
 
     psDoc = xmlNewDoc(BAD_CAST "1.0");
 
-    psRootNode = msOWSCommonExceptionReport(psNsOws, OWS_1_1_0, schemasLocation, version, msOWSGetLanguage(map, "exception"), exceptionCode, locator, errorMessage);
+    psRootNode = msOWSCommonExceptionReport(psNsOws, OWS_1_0_0, schemasLocation, version, msOWSGetLanguage(map, "exception"), exceptionCode, locator, errorMessage);
 
     xmlDocSetRootElement(psDoc, psRootNode);
 
-    psNsOws = xmlNewNs(psRootNode, BAD_CAST "http://www.opengis.net/ows/1.1", BAD_CAST "ows");
+    psNsOws = xmlNewNs(psRootNode, BAD_CAST "http://www.opengis.net/ows", BAD_CAST "ows");
 
-    msIO_printf("Content-type: text/xml%c%c",10,10);
-    xmlDocDumpFormatMemoryEnc(psDoc, &buffer, &size, "ISO-8859-1", 1);
+    if (encoding)
+        msIO_printf("Content-type: text/xml; charset=%s%c%c", encoding,10,10);
+    else
+        msIO_printf("Content-type: text/xml%c%c",10,10);
+
+    xmlDocDumpFormatMemoryEnc(psDoc, &buffer, &size, (encoding ? encoding : "ISO-8859-1"), 1);
     
     msIO_printf("%s", buffer);
 
@@ -165,7 +170,7 @@ static xmlNodePtr msWFSDumpLayer11(mapObj *map, layerObj *lp, xmlNsPtr psNsOws)
     {
         xmlNewChild(psRootNode, psNsOws, BAD_CAST "WGS84BoundingBox", NULL);
         xmlAddSibling(psNode,
-                      xmlNewComment(BAD_CAST "WARNING: Mandatory WGS84BoundingBox could not be established for this layer.  Consider setting LAYER.EXTENT or wfs_extent metadata."));
+                      xmlNewComment(BAD_CAST "WARNING: Optional WGS84BoundingBox could not be established for this layer.  Consider setting the EXTENT in the LAYER object, or wfs_extent metadata. Also check that your data exists in the DATA statement"));
     }
 
     value = msOWSLookupMetadata(&(lp->metadata), "FO", "metadataurl_href");
@@ -209,6 +214,8 @@ int msWFSGetCapabilities11(mapObj *map, wfsParamsObj *params,
     char *xsi_schemaLocation = NULL;
 
     char *script_url=NULL, *script_url_encoded=NULL;
+    const char *value = NULL;
+    const char *encoding;
 
     xmlChar *buffer = NULL;
     int size = 0, i;
@@ -222,6 +229,8 @@ int msWFSGetCapabilities11(mapObj *map, wfsParamsObj *params,
 
     updatesequence = msOWSLookupMetadata(&(map->web.metadata), "FO", "updatesequence");
 
+    encoding = msOWSLookupMetadata(&(map->web.metadata), "FO", "encoding");
+
     if (params->pszUpdateSequence != NULL) {
       i = msOWSNegotiateUpdateSequence(params->pszUpdateSequence, updatesequence);
       if (i == 0) { /* current */
@@ -331,7 +340,7 @@ int msWFSGetCapabilities11(mapObj *map, wfsParamsObj *params,
                                                                   "XMLSCHEMA,text/xml; subtype=gml/2.1.2,text/xml; subtype=gml/3.1.1"));
 
 /* -------------------------------------------------------------------- */
-/*      GetFeatureType                                                  */
+/*      GetFeature                                                      */
 /* -------------------------------------------------------------------- */
       psNode = xmlAddChild(psMainNode, 
                           msOWSCommonOperationsMetadataOperation(psNsOws,psNsXLink,"GetFeature", 
@@ -351,6 +360,14 @@ int msWFSGetCapabilities11(mapObj *map, wfsParamsObj *params,
                                                                   "Parameter", "outputFormat", 
                                                                   "text/xml; subtype=gml/3.1.1"));
 
+     value = msOWSLookupMetadata(&(map->web.metadata), "FO", "maxfeatures");
+
+    if (value) {
+         xmlAddChild(psMainNode, msOWSCommonOperationsMetadataDomainType(ows_version, psNsOws,
+                                                                  "Constraint", "DefaultMaxFeatures",
+                                                                  (char *)value));
+    }
+
 /* -------------------------------------------------------------------- */
 /*      FeatureTypeList                                                 */
 /* -------------------------------------------------------------------- */
@@ -387,11 +404,14 @@ int msWFSGetCapabilities11(mapObj *map, wfsParamsObj *params,
     if( msIO_needBinaryStdout() == MS_FAILURE )
         return MS_FAILURE;
      
-    msIO_printf("Content-type: text/xml%c%c",10,10);
+    if (encoding)
+        msIO_printf("Content-type: text/xml; charset=%s%c%c", encoding,10,10);
+    else
+        msIO_printf("Content-type: text/xml%c%c",10,10);
     
     context = msIO_getHandler(stdout);
 
-    xmlDocDumpFormatMemoryEnc(psDoc, &buffer, &size, "ISO-8859-1", 1);
+    xmlDocDumpFormatMemoryEnc(psDoc, &buffer, &size, (encoding ? encoding : "ISO-8859-1"), 1);
     msIO_contextWrite(context, buffer, size);
     xmlFree(buffer);
 
