@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mapwfslayer.c 9471 2009-10-16 17:04:04Z assefa $
+ * $Id: mapwfslayer.c 10264 2010-06-29 16:20:53Z dmorissette $
  *
  * Project:  MapServer
  * Purpose:  Implementation of WFS CONNECTIONTYPE - client to WFS servers
@@ -37,7 +37,7 @@
 #include <process.h>
 #endif
 
-MS_CVSID("$Id: mapwfslayer.c 9471 2009-10-16 17:04:04Z assefa $")
+MS_CVSID("$Id: mapwfslayer.c 10264 2010-06-29 16:20:53Z dmorissette $")
 
 #define WFS_V_0_0_14  14
 #define WFS_V_1_0_0  100
@@ -156,9 +156,14 @@ static wfsParamsObj *msBuildRequestParams(mapObj *map, layerObj *lp,
     pszTmp = msOWSLookupMetadata(&(lp->metadata), "FO", "filter");
     if (pszTmp && strlen(pszTmp) > 0)
     {
-        psParams->pszFilter = malloc(sizeof(char)*(strlen(pszTmp)+17+1));
-        sprintf(psParams->pszFilter, "<Filter>%s</Filter>", pszTmp);
-        /* <Filter xmlns=\"http://www.opengis.net/ogc\" xmlns:gml=\"http://www.opengis.net/gml\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"http://www.opengis.net/ogc ../filter/1.0.0/filter.xsd http://www.opengis.net/gml../gml/2.1/geometry.xsd\"> */
+        if (strstr(pszTmp, "<Filter>") !=NULL || strstr(pszTmp, "<ogc:Filter>") != NULL)
+          psParams->pszFilter = strdup(pszTmp);
+        else
+        {
+            psParams->pszFilter = msStringConcatenate(psParams->pszFilter, "<ogc:Filter>");
+            psParams->pszFilter = msStringConcatenate(psParams->pszFilter, pszTmp);
+            psParams->pszFilter = msStringConcatenate(psParams->pszFilter, "</ogc:Filter>");
+        }
     }
 
      pszTmp = msOWSLookupMetadata(&(lp->metadata), "FO", "maxfeatures");
@@ -246,40 +251,41 @@ static char *msBuildWFSLayerPostRequest(mapObj *map, layerObj *lp,
     else
     {
         pszFilter = (char *)malloc(sizeof(char)*500);
-        sprintf(pszFilter, "<Filter>\n"
-"<BBOX>\n"
-"<PropertyName>Geometry</PropertyName>\n"
-"<Box>\n"
-"<coordinates>%f,%f %f,%f</coordinates>\n"
-"</Box>\n"
-"</BBOX>\n"
-"</Filter>",bbox->minx, bbox->miny, bbox->maxx, bbox->maxy);
+        sprintf(pszFilter, "<ogc:Filter>\n"
+"<ogc:BBOX>\n"
+"<ogc:PropertyName>Geometry</ogc:PropertyName>\n"
+"<gml:Box>\n"
+"<gml:coordinates>%f,%f %f,%f</gml:coordinates>\n"
+"</gml:Box>\n"
+"</ogc:BBOX>\n"
+"</ogc:Filter>",bbox->minx, bbox->miny, bbox->maxx, bbox->maxy);
     }
 
     pszPostReq = (char *)malloc(sizeof(char)*(strlen(pszFilter)+500));
     if (psParams->nMaxFeatures > 0)
       sprintf(pszPostReq, "<?xml version=\"1.0\" ?>\n"
-"<GetFeature\n"
+"<wfs:GetFeature\n"
 "service=\"WFS\"\n"
 "version=\"1.0.0\"\n"
 "maxFeatures=\"%d\"\n"
 "outputFormat=\"GML2\">\n"
-"<Query typeName=\"%s\">\n"
+"<wfs:Query typeName=\"%s\">\n"
 "%s"
-"</Query>\n"
-"</GetFeature>\n", psParams->nMaxFeatures, psParams->pszTypeName, pszFilter);
+"</wfs:Query>\n"
+"</wfs:GetFeature>\n", psParams->nMaxFeatures, psParams->pszTypeName, pszFilter);
     else
       sprintf(pszPostReq, "<?xml version=\"1.0\" ?>\n"
-"<GetFeature\n"
+"<wfs:GetFeature\n"
 "service=\"WFS\"\n"
 "version=\"1.0.0\"\n"
 "outputFormat=\"GML2\">\n"
-"<Query typeName=\"%s\">\n"
+"<wfs:Query typeName=\"%s\">\n"
 "%s"
-"</Query>\n"
-"</GetFeature>\n", psParams->pszTypeName, pszFilter);
-    if (psParams->pszFilter == NULL)
-      free(pszFilter);
+"</wfs:Query>\n"
+"</wfs:GetFeature>\n", psParams->pszTypeName, pszFilter);
+    
+    if (psParams->pszFilter != NULL)
+      msFree(pszFilter);
 
 
     return pszPostReq; 
@@ -514,7 +520,6 @@ int msPrepareWFSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
     int nTimeout;
     int nStatus = MS_SUCCESS;
     msWFSLayerInfo *psInfo = NULL;
-    char *pszHashFileName = NULL;
     int bPostRequest = 0;
     wfsParamsObj *psParams = NULL;
     char *pszHTTPCookieData = NULL;
@@ -646,23 +651,12 @@ int msPrepareWFSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
     }
 
     /* We'll store the remote server's response to a tmp file. */
-    if (bPostRequest)
-    {
-        char *pszPostTmpName = NULL;
-        pszPostTmpName = (char *)malloc(sizeof(char)*(strlen(pszURL)+128));
-        sprintf(pszPostTmpName,"%s%ld%d",
-                pszURL, (long)time(NULL), (int)getpid());
-        pszHashFileName = msHashString(pszPostTmpName);
-        free(pszPostTmpName);
-    }
-    else
-      pszHashFileName = msHashString(pszURL);
-    pszURL = NULL;
-    
-    pasReqInfo[(*numRequests)].pszOutputFile =  
-        msOWSBuildURLFilename(map->web.imagepath, 
-                              pszHashFileName,".tmp.gml");
-    free(pszHashFileName);
+    pasReqInfo[(*numRequests)].pszOutputFile = msTmpFile(map->mappath, 
+                                                 map->web.imagepath, ".tmp.gml"); 
+    /* TODO: Implement Caching of GML responses. There was an older caching 
+     * method, but it suffered from a race condition. See #3137.
+     */ 
+
     pasReqInfo[(*numRequests)].pszHTTPCookieData = pszHTTPCookieData;
     pszHTTPCookieData = NULL;
     pasReqInfo[(*numRequests)].nStatus = 0;
@@ -946,6 +940,43 @@ int msWFSLayerGetShape(layerObj *layer, shapeObj *shape, int tile,
 
 }
 
+/**********************************************************************
+ *                          msWFSLayerResultGetShape()
+ *
+ **********************************************************************/
+int msWFSLayerResultGetShape(layerObj *layer, shapeObj *shape, int tile, 
+                             long record)
+{
+#ifdef USE_WFS_LYR
+    msWFSLayerInfo* psInfo = NULL;
+
+    if(layer != NULL && layer->wfslayerinfo != NULL)
+    	psInfo = (msWFSLayerInfo*)layer->wfslayerinfo;
+    else
+    {
+	msSetError(MS_WFSERR, "Layer is not opened.", "msWFSLayerResultGetShape()");
+	return MS_FAILURE;
+    }
+
+    if(psInfo->bLayerHasValidGML)
+      return msOGRLayerResultGetShape(layer, shape, tile, record);
+    else
+    {
+          /* Layer is successful, but there is no data to process */
+        msFreeShape(shape);
+        shape->type = MS_SHAPE_NULL;
+        return MS_FAILURE;
+    }
+#else
+/* ------------------------------------------------------------------
+ * WFS CONNECTION Support not included...
+ * ------------------------------------------------------------------ */
+  msSetError(MS_WFSCONNERR, "WFS CLIENT CONNECTION support is not available.", 
+             "msWFSLayerResultGetShape()");
+  return(MS_FAILURE);
+#endif /* USE_WFS_LYR */
+
+}
 
 
 
@@ -1333,7 +1364,7 @@ msWFSLayerInitializeVirtualTable(layerObj *layer)
     layer->vtable->LayerIsOpen = msWFSLayerIsOpen;
     layer->vtable->LayerWhichShapes = msWFSLayerWhichShapes;
     layer->vtable->LayerNextShape = msWFSLayerNextShape;
-    layer->vtable->LayerResultsGetShape = msWFSLayerGetShape; 
+    layer->vtable->LayerResultsGetShape = msWFSLayerResultGetShape; 
     layer->vtable->LayerGetShape = msWFSLayerGetShape;
     layer->vtable->LayerClose = msWFSLayerClose;
     layer->vtable->LayerGetItems = msWFSLayerGetItems;
