@@ -1,5 +1,5 @@
 /*****************************************************************************
- * $Id: mapwmslayer.c 9281 2009-08-28 21:36:25Z warmerdam $
+ * $Id: mapwmslayer.c 10138 2010-05-06 15:09:22Z pramsey $
  *
  * Project:  MapServer
  * Purpose:  Implementation of WMS CONNECTIONTYPE - client to WMS servers
@@ -1346,6 +1346,10 @@ int msDrawWMSLayerLow(int nLayerId, httpRequestObj *pasReqInfo,
         msOWSLookupMetadata(&(lp->metadata), "MO", "sld_url"))        
       lp->numclasses = 0;
 
+    /* ensure the file connection is closed right away after the layer */ 
+    /* is rendered */ 
+    msLayerSetProcessingKey( lp, "CLOSE_CONNECTION", "NORMAL");
+
     if (lp->data) free(lp->data);
     lp->data =  strdup(pasReqInfo[iReq].pszOutputFile);
 
