@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: shp2img.c 8489 2009-02-03 20:35:24Z tomkralidis $
+ * $Id: shp2img.c 9518 2009-11-04 13:53:23Z aboudreault $
  *
  * Project:  MapServer
  * Purpose:  Commandline .map rendering utility, mostly for testing.
@@ -30,7 +30,7 @@
 #include "mapserver.h"
 #include "maptime.h"
 
-MS_CVSID("$Id: shp2img.c 8489 2009-02-03 20:35:24Z tomkralidis $")
+MS_CVSID("$Id: shp2img.c 9518 2009-11-04 13:53:23Z aboudreault $")
 
 int main(int argc, char *argv[])
 {
@@ -133,6 +133,7 @@ int main(int argc, char *argv[])
         msCleanup();
 	exit(0);
       }
+      msApplyDefaultSubstitutions(map);
     }
   }
 
@@ -262,6 +263,8 @@ int main(int argc, char *argv[])
             break;
           }
           else {
+            if (invalid_layer)
+              free(invalid_layer);
             invalid_layer = strdup(layers[j]);
           }
         }
@@ -270,6 +273,8 @@ int main(int argc, char *argv[])
           msCleanup();
           exit(0);
         }
+        if (invalid_layer)
+          free(invalid_layer);
       }
 
       for(j=0; j<map->numlayers; j++) {
