@@ -1,7 +1,10 @@
 /******************************************************************************
+ * $Id: sortshp.c 7418 2008-02-29 00:02:49Z nsavard $
  *
  * Project:  MapServer
- * Purpose:  Commandline utility to ... sort shapes?  Not sure what its for.
+ * Purpose:  Command line utility to sort a shapefile based on a single 
+ *           attribute in ascending or decending order. Useful for 
+ *           prioritizing drawing or labeling of shapes.
  * Author:   Steve Lime and the MapServer team.
  *
  ******************************************************************************
@@ -33,7 +36,7 @@
 
 #include "mapserver.h"
 
-MS_CVSID("$Id: sortshp.c 6428 2007-07-31 19:58:45Z dmorissette $")
+MS_CVSID("$Id: sortshp.c 7418 2008-02-29 00:02:49Z nsavard $")
 
 typedef struct {
   double number;
